require 'minitest/unit'

require 'metadata/linux/LinuxUtils'

class TestLinuxUtils < MiniTest::Unit::TestCase
  #  puts "  OCTAL_TO_PERMISSIONS = {"
  #  (0..07777).each do |o|
  #    puts "    '%04o' => '%s'," % [o, MiqLinux::Utils.octal_to_permissions(o)]
  #  end
  #  puts "  }"

  OCTAL_TO_PERMISSIONS = {
    '0000' => '---------',
    '0001' => '--------x',
    '0002' => '-------w-',
    '0003' => '-------wx',
    '0004' => '------r--',
    '0005' => '------r-x',
    '0006' => '------rw-',
    '0007' => '------rwx',
    '0010' => '-----x---',
    '0011' => '-----x--x',
    '0012' => '-----x-w-',
    '0013' => '-----x-wx',
    '0014' => '-----xr--',
    '0015' => '-----xr-x',
    '0016' => '-----xrw-',
    '0017' => '-----xrwx',
    '0020' => '----w----',
    '0021' => '----w---x',
    '0022' => '----w--w-',
    '0023' => '----w--wx',
    '0024' => '----w-r--',
    '0025' => '----w-r-x',
    '0026' => '----w-rw-',
    '0027' => '----w-rwx',
    '0030' => '----wx---',
    '0031' => '----wx--x',
    '0032' => '----wx-w-',
    '0033' => '----wx-wx',
    '0034' => '----wxr--',
    '0035' => '----wxr-x',
    '0036' => '----wxrw-',
    '0037' => '----wxrwx',
    '0040' => '---r-----',
    '0041' => '---r----x',
    '0042' => '---r---w-',
    '0043' => '---r---wx',
    '0044' => '---r--r--',
    '0045' => '---r--r-x',
    '0046' => '---r--rw-',
    '0047' => '---r--rwx',
    '0050' => '---r-x---',
    '0051' => '---r-x--x',
    '0052' => '---r-x-w-',
    '0053' => '---r-x-wx',
    '0054' => '---r-xr--',
    '0055' => '---r-xr-x',
    '0056' => '---r-xrw-',
    '0057' => '---r-xrwx',
    '0060' => '---rw----',
    '0061' => '---rw---x',
    '0062' => '---rw--w-',
    '0063' => '---rw--wx',
    '0064' => '---rw-r--',
    '0065' => '---rw-r-x',
    '0066' => '---rw-rw-',
    '0067' => '---rw-rwx',
    '0070' => '---rwx---',
    '0071' => '---rwx--x',
    '0072' => '---rwx-w-',
    '0073' => '---rwx-wx',
    '0074' => '---rwxr--',
    '0075' => '---rwxr-x',
    '0076' => '---rwxrw-',
    '0077' => '---rwxrwx',
    '0100' => '--x------',
    '0101' => '--x-----x',
    '0102' => '--x----w-',
    '0103' => '--x----wx',
    '0104' => '--x---r--',
    '0105' => '--x---r-x',
    '0106' => '--x---rw-',
    '0107' => '--x---rwx',
    '0110' => '--x--x---',
    '0111' => '--x--x--x',
    '0112' => '--x--x-w-',
    '0113' => '--x--x-wx',
    '0114' => '--x--xr--',
    '0115' => '--x--xr-x',
    '0116' => '--x--xrw-',
    '0117' => '--x--xrwx',
    '0120' => '--x-w----',
    '0121' => '--x-w---x',
    '0122' => '--x-w--w-',
    '0123' => '--x-w--wx',
    '0124' => '--x-w-r--',
    '0125' => '--x-w-r-x',
    '0126' => '--x-w-rw-',
    '0127' => '--x-w-rwx',
    '0130' => '--x-wx---',
    '0131' => '--x-wx--x',
    '0132' => '--x-wx-w-',
    '0133' => '--x-wx-wx',
    '0134' => '--x-wxr--',
    '0135' => '--x-wxr-x',
    '0136' => '--x-wxrw-',
    '0137' => '--x-wxrwx',
    '0140' => '--xr-----',
    '0141' => '--xr----x',
    '0142' => '--xr---w-',
    '0143' => '--xr---wx',
    '0144' => '--xr--r--',
    '0145' => '--xr--r-x',
    '0146' => '--xr--rw-',
    '0147' => '--xr--rwx',
    '0150' => '--xr-x---',
    '0151' => '--xr-x--x',
    '0152' => '--xr-x-w-',
    '0153' => '--xr-x-wx',
    '0154' => '--xr-xr--',
    '0155' => '--xr-xr-x',
    '0156' => '--xr-xrw-',
    '0157' => '--xr-xrwx',
    '0160' => '--xrw----',
    '0161' => '--xrw---x',
    '0162' => '--xrw--w-',
    '0163' => '--xrw--wx',
    '0164' => '--xrw-r--',
    '0165' => '--xrw-r-x',
    '0166' => '--xrw-rw-',
    '0167' => '--xrw-rwx',
    '0170' => '--xrwx---',
    '0171' => '--xrwx--x',
    '0172' => '--xrwx-w-',
    '0173' => '--xrwx-wx',
    '0174' => '--xrwxr--',
    '0175' => '--xrwxr-x',
    '0176' => '--xrwxrw-',
    '0177' => '--xrwxrwx',
    '0200' => '-w-------',
    '0201' => '-w------x',
    '0202' => '-w-----w-',
    '0203' => '-w-----wx',
    '0204' => '-w----r--',
    '0205' => '-w----r-x',
    '0206' => '-w----rw-',
    '0207' => '-w----rwx',
    '0210' => '-w---x---',
    '0211' => '-w---x--x',
    '0212' => '-w---x-w-',
    '0213' => '-w---x-wx',
    '0214' => '-w---xr--',
    '0215' => '-w---xr-x',
    '0216' => '-w---xrw-',
    '0217' => '-w---xrwx',
    '0220' => '-w--w----',
    '0221' => '-w--w---x',
    '0222' => '-w--w--w-',
    '0223' => '-w--w--wx',
    '0224' => '-w--w-r--',
    '0225' => '-w--w-r-x',
    '0226' => '-w--w-rw-',
    '0227' => '-w--w-rwx',
    '0230' => '-w--wx---',
    '0231' => '-w--wx--x',
    '0232' => '-w--wx-w-',
    '0233' => '-w--wx-wx',
    '0234' => '-w--wxr--',
    '0235' => '-w--wxr-x',
    '0236' => '-w--wxrw-',
    '0237' => '-w--wxrwx',
    '0240' => '-w-r-----',
    '0241' => '-w-r----x',
    '0242' => '-w-r---w-',
    '0243' => '-w-r---wx',
    '0244' => '-w-r--r--',
    '0245' => '-w-r--r-x',
    '0246' => '-w-r--rw-',
    '0247' => '-w-r--rwx',
    '0250' => '-w-r-x---',
    '0251' => '-w-r-x--x',
    '0252' => '-w-r-x-w-',
    '0253' => '-w-r-x-wx',
    '0254' => '-w-r-xr--',
    '0255' => '-w-r-xr-x',
    '0256' => '-w-r-xrw-',
    '0257' => '-w-r-xrwx',
    '0260' => '-w-rw----',
    '0261' => '-w-rw---x',
    '0262' => '-w-rw--w-',
    '0263' => '-w-rw--wx',
    '0264' => '-w-rw-r--',
    '0265' => '-w-rw-r-x',
    '0266' => '-w-rw-rw-',
    '0267' => '-w-rw-rwx',
    '0270' => '-w-rwx---',
    '0271' => '-w-rwx--x',
    '0272' => '-w-rwx-w-',
    '0273' => '-w-rwx-wx',
    '0274' => '-w-rwxr--',
    '0275' => '-w-rwxr-x',
    '0276' => '-w-rwxrw-',
    '0277' => '-w-rwxrwx',
    '0300' => '-wx------',
    '0301' => '-wx-----x',
    '0302' => '-wx----w-',
    '0303' => '-wx----wx',
    '0304' => '-wx---r--',
    '0305' => '-wx---r-x',
    '0306' => '-wx---rw-',
    '0307' => '-wx---rwx',
    '0310' => '-wx--x---',
    '0311' => '-wx--x--x',
    '0312' => '-wx--x-w-',
    '0313' => '-wx--x-wx',
    '0314' => '-wx--xr--',
    '0315' => '-wx--xr-x',
    '0316' => '-wx--xrw-',
    '0317' => '-wx--xrwx',
    '0320' => '-wx-w----',
    '0321' => '-wx-w---x',
    '0322' => '-wx-w--w-',
    '0323' => '-wx-w--wx',
    '0324' => '-wx-w-r--',
    '0325' => '-wx-w-r-x',
    '0326' => '-wx-w-rw-',
    '0327' => '-wx-w-rwx',
    '0330' => '-wx-wx---',
    '0331' => '-wx-wx--x',
    '0332' => '-wx-wx-w-',
    '0333' => '-wx-wx-wx',
    '0334' => '-wx-wxr--',
    '0335' => '-wx-wxr-x',
    '0336' => '-wx-wxrw-',
    '0337' => '-wx-wxrwx',
    '0340' => '-wxr-----',
    '0341' => '-wxr----x',
    '0342' => '-wxr---w-',
    '0343' => '-wxr---wx',
    '0344' => '-wxr--r--',
    '0345' => '-wxr--r-x',
    '0346' => '-wxr--rw-',
    '0347' => '-wxr--rwx',
    '0350' => '-wxr-x---',
    '0351' => '-wxr-x--x',
    '0352' => '-wxr-x-w-',
    '0353' => '-wxr-x-wx',
    '0354' => '-wxr-xr--',
    '0355' => '-wxr-xr-x',
    '0356' => '-wxr-xrw-',
    '0357' => '-wxr-xrwx',
    '0360' => '-wxrw----',
    '0361' => '-wxrw---x',
    '0362' => '-wxrw--w-',
    '0363' => '-wxrw--wx',
    '0364' => '-wxrw-r--',
    '0365' => '-wxrw-r-x',
    '0366' => '-wxrw-rw-',
    '0367' => '-wxrw-rwx',
    '0370' => '-wxrwx---',
    '0371' => '-wxrwx--x',
    '0372' => '-wxrwx-w-',
    '0373' => '-wxrwx-wx',
    '0374' => '-wxrwxr--',
    '0375' => '-wxrwxr-x',
    '0376' => '-wxrwxrw-',
    '0377' => '-wxrwxrwx',
    '0400' => 'r--------',
    '0401' => 'r-------x',
    '0402' => 'r------w-',
    '0403' => 'r------wx',
    '0404' => 'r-----r--',
    '0405' => 'r-----r-x',
    '0406' => 'r-----rw-',
    '0407' => 'r-----rwx',
    '0410' => 'r----x---',
    '0411' => 'r----x--x',
    '0412' => 'r----x-w-',
    '0413' => 'r----x-wx',
    '0414' => 'r----xr--',
    '0415' => 'r----xr-x',
    '0416' => 'r----xrw-',
    '0417' => 'r----xrwx',
    '0420' => 'r---w----',
    '0421' => 'r---w---x',
    '0422' => 'r---w--w-',
    '0423' => 'r---w--wx',
    '0424' => 'r---w-r--',
    '0425' => 'r---w-r-x',
    '0426' => 'r---w-rw-',
    '0427' => 'r---w-rwx',
    '0430' => 'r---wx---',
    '0431' => 'r---wx--x',
    '0432' => 'r---wx-w-',
    '0433' => 'r---wx-wx',
    '0434' => 'r---wxr--',
    '0435' => 'r---wxr-x',
    '0436' => 'r---wxrw-',
    '0437' => 'r---wxrwx',
    '0440' => 'r--r-----',
    '0441' => 'r--r----x',
    '0442' => 'r--r---w-',
    '0443' => 'r--r---wx',
    '0444' => 'r--r--r--',
    '0445' => 'r--r--r-x',
    '0446' => 'r--r--rw-',
    '0447' => 'r--r--rwx',
    '0450' => 'r--r-x---',
    '0451' => 'r--r-x--x',
    '0452' => 'r--r-x-w-',
    '0453' => 'r--r-x-wx',
    '0454' => 'r--r-xr--',
    '0455' => 'r--r-xr-x',
    '0456' => 'r--r-xrw-',
    '0457' => 'r--r-xrwx',
    '0460' => 'r--rw----',
    '0461' => 'r--rw---x',
    '0462' => 'r--rw--w-',
    '0463' => 'r--rw--wx',
    '0464' => 'r--rw-r--',
    '0465' => 'r--rw-r-x',
    '0466' => 'r--rw-rw-',
    '0467' => 'r--rw-rwx',
    '0470' => 'r--rwx---',
    '0471' => 'r--rwx--x',
    '0472' => 'r--rwx-w-',
    '0473' => 'r--rwx-wx',
    '0474' => 'r--rwxr--',
    '0475' => 'r--rwxr-x',
    '0476' => 'r--rwxrw-',
    '0477' => 'r--rwxrwx',
    '0500' => 'r-x------',
    '0501' => 'r-x-----x',
    '0502' => 'r-x----w-',
    '0503' => 'r-x----wx',
    '0504' => 'r-x---r--',
    '0505' => 'r-x---r-x',
    '0506' => 'r-x---rw-',
    '0507' => 'r-x---rwx',
    '0510' => 'r-x--x---',
    '0511' => 'r-x--x--x',
    '0512' => 'r-x--x-w-',
    '0513' => 'r-x--x-wx',
    '0514' => 'r-x--xr--',
    '0515' => 'r-x--xr-x',
    '0516' => 'r-x--xrw-',
    '0517' => 'r-x--xrwx',
    '0520' => 'r-x-w----',
    '0521' => 'r-x-w---x',
    '0522' => 'r-x-w--w-',
    '0523' => 'r-x-w--wx',
    '0524' => 'r-x-w-r--',
    '0525' => 'r-x-w-r-x',
    '0526' => 'r-x-w-rw-',
    '0527' => 'r-x-w-rwx',
    '0530' => 'r-x-wx---',
    '0531' => 'r-x-wx--x',
    '0532' => 'r-x-wx-w-',
    '0533' => 'r-x-wx-wx',
    '0534' => 'r-x-wxr--',
    '0535' => 'r-x-wxr-x',
    '0536' => 'r-x-wxrw-',
    '0537' => 'r-x-wxrwx',
    '0540' => 'r-xr-----',
    '0541' => 'r-xr----x',
    '0542' => 'r-xr---w-',
    '0543' => 'r-xr---wx',
    '0544' => 'r-xr--r--',
    '0545' => 'r-xr--r-x',
    '0546' => 'r-xr--rw-',
    '0547' => 'r-xr--rwx',
    '0550' => 'r-xr-x---',
    '0551' => 'r-xr-x--x',
    '0552' => 'r-xr-x-w-',
    '0553' => 'r-xr-x-wx',
    '0554' => 'r-xr-xr--',
    '0555' => 'r-xr-xr-x',
    '0556' => 'r-xr-xrw-',
    '0557' => 'r-xr-xrwx',
    '0560' => 'r-xrw----',
    '0561' => 'r-xrw---x',
    '0562' => 'r-xrw--w-',
    '0563' => 'r-xrw--wx',
    '0564' => 'r-xrw-r--',
    '0565' => 'r-xrw-r-x',
    '0566' => 'r-xrw-rw-',
    '0567' => 'r-xrw-rwx',
    '0570' => 'r-xrwx---',
    '0571' => 'r-xrwx--x',
    '0572' => 'r-xrwx-w-',
    '0573' => 'r-xrwx-wx',
    '0574' => 'r-xrwxr--',
    '0575' => 'r-xrwxr-x',
    '0576' => 'r-xrwxrw-',
    '0577' => 'r-xrwxrwx',
    '0600' => 'rw-------',
    '0601' => 'rw------x',
    '0602' => 'rw-----w-',
    '0603' => 'rw-----wx',
    '0604' => 'rw----r--',
    '0605' => 'rw----r-x',
    '0606' => 'rw----rw-',
    '0607' => 'rw----rwx',
    '0610' => 'rw---x---',
    '0611' => 'rw---x--x',
    '0612' => 'rw---x-w-',
    '0613' => 'rw---x-wx',
    '0614' => 'rw---xr--',
    '0615' => 'rw---xr-x',
    '0616' => 'rw---xrw-',
    '0617' => 'rw---xrwx',
    '0620' => 'rw--w----',
    '0621' => 'rw--w---x',
    '0622' => 'rw--w--w-',
    '0623' => 'rw--w--wx',
    '0624' => 'rw--w-r--',
    '0625' => 'rw--w-r-x',
    '0626' => 'rw--w-rw-',
    '0627' => 'rw--w-rwx',
    '0630' => 'rw--wx---',
    '0631' => 'rw--wx--x',
    '0632' => 'rw--wx-w-',
    '0633' => 'rw--wx-wx',
    '0634' => 'rw--wxr--',
    '0635' => 'rw--wxr-x',
    '0636' => 'rw--wxrw-',
    '0637' => 'rw--wxrwx',
    '0640' => 'rw-r-----',
    '0641' => 'rw-r----x',
    '0642' => 'rw-r---w-',
    '0643' => 'rw-r---wx',
    '0644' => 'rw-r--r--',
    '0645' => 'rw-r--r-x',
    '0646' => 'rw-r--rw-',
    '0647' => 'rw-r--rwx',
    '0650' => 'rw-r-x---',
    '0651' => 'rw-r-x--x',
    '0652' => 'rw-r-x-w-',
    '0653' => 'rw-r-x-wx',
    '0654' => 'rw-r-xr--',
    '0655' => 'rw-r-xr-x',
    '0656' => 'rw-r-xrw-',
    '0657' => 'rw-r-xrwx',
    '0660' => 'rw-rw----',
    '0661' => 'rw-rw---x',
    '0662' => 'rw-rw--w-',
    '0663' => 'rw-rw--wx',
    '0664' => 'rw-rw-r--',
    '0665' => 'rw-rw-r-x',
    '0666' => 'rw-rw-rw-',
    '0667' => 'rw-rw-rwx',
    '0670' => 'rw-rwx---',
    '0671' => 'rw-rwx--x',
    '0672' => 'rw-rwx-w-',
    '0673' => 'rw-rwx-wx',
    '0674' => 'rw-rwxr--',
    '0675' => 'rw-rwxr-x',
    '0676' => 'rw-rwxrw-',
    '0677' => 'rw-rwxrwx',
    '0700' => 'rwx------',
    '0701' => 'rwx-----x',
    '0702' => 'rwx----w-',
    '0703' => 'rwx----wx',
    '0704' => 'rwx---r--',
    '0705' => 'rwx---r-x',
    '0706' => 'rwx---rw-',
    '0707' => 'rwx---rwx',
    '0710' => 'rwx--x---',
    '0711' => 'rwx--x--x',
    '0712' => 'rwx--x-w-',
    '0713' => 'rwx--x-wx',
    '0714' => 'rwx--xr--',
    '0715' => 'rwx--xr-x',
    '0716' => 'rwx--xrw-',
    '0717' => 'rwx--xrwx',
    '0720' => 'rwx-w----',
    '0721' => 'rwx-w---x',
    '0722' => 'rwx-w--w-',
    '0723' => 'rwx-w--wx',
    '0724' => 'rwx-w-r--',
    '0725' => 'rwx-w-r-x',
    '0726' => 'rwx-w-rw-',
    '0727' => 'rwx-w-rwx',
    '0730' => 'rwx-wx---',
    '0731' => 'rwx-wx--x',
    '0732' => 'rwx-wx-w-',
    '0733' => 'rwx-wx-wx',
    '0734' => 'rwx-wxr--',
    '0735' => 'rwx-wxr-x',
    '0736' => 'rwx-wxrw-',
    '0737' => 'rwx-wxrwx',
    '0740' => 'rwxr-----',
    '0741' => 'rwxr----x',
    '0742' => 'rwxr---w-',
    '0743' => 'rwxr---wx',
    '0744' => 'rwxr--r--',
    '0745' => 'rwxr--r-x',
    '0746' => 'rwxr--rw-',
    '0747' => 'rwxr--rwx',
    '0750' => 'rwxr-x---',
    '0751' => 'rwxr-x--x',
    '0752' => 'rwxr-x-w-',
    '0753' => 'rwxr-x-wx',
    '0754' => 'rwxr-xr--',
    '0755' => 'rwxr-xr-x',
    '0756' => 'rwxr-xrw-',
    '0757' => 'rwxr-xrwx',
    '0760' => 'rwxrw----',
    '0761' => 'rwxrw---x',
    '0762' => 'rwxrw--w-',
    '0763' => 'rwxrw--wx',
    '0764' => 'rwxrw-r--',
    '0765' => 'rwxrw-r-x',
    '0766' => 'rwxrw-rw-',
    '0767' => 'rwxrw-rwx',
    '0770' => 'rwxrwx---',
    '0771' => 'rwxrwx--x',
    '0772' => 'rwxrwx-w-',
    '0773' => 'rwxrwx-wx',
    '0774' => 'rwxrwxr--',
    '0775' => 'rwxrwxr-x',
    '0776' => 'rwxrwxrw-',
    '0777' => 'rwxrwxrwx',
    '1000' => '--------T',
    '1001' => '--------t',
    '1002' => '-------wT',
    '1003' => '-------wt',
    '1004' => '------r-T',
    '1005' => '------r-t',
    '1006' => '------rwT',
    '1007' => '------rwt',
    '1010' => '-----x--T',
    '1011' => '-----x--t',
    '1012' => '-----x-wT',
    '1013' => '-----x-wt',
    '1014' => '-----xr-T',
    '1015' => '-----xr-t',
    '1016' => '-----xrwT',
    '1017' => '-----xrwt',
    '1020' => '----w---T',
    '1021' => '----w---t',
    '1022' => '----w--wT',
    '1023' => '----w--wt',
    '1024' => '----w-r-T',
    '1025' => '----w-r-t',
    '1026' => '----w-rwT',
    '1027' => '----w-rwt',
    '1030' => '----wx--T',
    '1031' => '----wx--t',
    '1032' => '----wx-wT',
    '1033' => '----wx-wt',
    '1034' => '----wxr-T',
    '1035' => '----wxr-t',
    '1036' => '----wxrwT',
    '1037' => '----wxrwt',
    '1040' => '---r----T',
    '1041' => '---r----t',
    '1042' => '---r---wT',
    '1043' => '---r---wt',
    '1044' => '---r--r-T',
    '1045' => '---r--r-t',
    '1046' => '---r--rwT',
    '1047' => '---r--rwt',
    '1050' => '---r-x--T',
    '1051' => '---r-x--t',
    '1052' => '---r-x-wT',
    '1053' => '---r-x-wt',
    '1054' => '---r-xr-T',
    '1055' => '---r-xr-t',
    '1056' => '---r-xrwT',
    '1057' => '---r-xrwt',
    '1060' => '---rw---T',
    '1061' => '---rw---t',
    '1062' => '---rw--wT',
    '1063' => '---rw--wt',
    '1064' => '---rw-r-T',
    '1065' => '---rw-r-t',
    '1066' => '---rw-rwT',
    '1067' => '---rw-rwt',
    '1070' => '---rwx--T',
    '1071' => '---rwx--t',
    '1072' => '---rwx-wT',
    '1073' => '---rwx-wt',
    '1074' => '---rwxr-T',
    '1075' => '---rwxr-t',
    '1076' => '---rwxrwT',
    '1077' => '---rwxrwt',
    '1100' => '--x-----T',
    '1101' => '--x-----t',
    '1102' => '--x----wT',
    '1103' => '--x----wt',
    '1104' => '--x---r-T',
    '1105' => '--x---r-t',
    '1106' => '--x---rwT',
    '1107' => '--x---rwt',
    '1110' => '--x--x--T',
    '1111' => '--x--x--t',
    '1112' => '--x--x-wT',
    '1113' => '--x--x-wt',
    '1114' => '--x--xr-T',
    '1115' => '--x--xr-t',
    '1116' => '--x--xrwT',
    '1117' => '--x--xrwt',
    '1120' => '--x-w---T',
    '1121' => '--x-w---t',
    '1122' => '--x-w--wT',
    '1123' => '--x-w--wt',
    '1124' => '--x-w-r-T',
    '1125' => '--x-w-r-t',
    '1126' => '--x-w-rwT',
    '1127' => '--x-w-rwt',
    '1130' => '--x-wx--T',
    '1131' => '--x-wx--t',
    '1132' => '--x-wx-wT',
    '1133' => '--x-wx-wt',
    '1134' => '--x-wxr-T',
    '1135' => '--x-wxr-t',
    '1136' => '--x-wxrwT',
    '1137' => '--x-wxrwt',
    '1140' => '--xr----T',
    '1141' => '--xr----t',
    '1142' => '--xr---wT',
    '1143' => '--xr---wt',
    '1144' => '--xr--r-T',
    '1145' => '--xr--r-t',
    '1146' => '--xr--rwT',
    '1147' => '--xr--rwt',
    '1150' => '--xr-x--T',
    '1151' => '--xr-x--t',
    '1152' => '--xr-x-wT',
    '1153' => '--xr-x-wt',
    '1154' => '--xr-xr-T',
    '1155' => '--xr-xr-t',
    '1156' => '--xr-xrwT',
    '1157' => '--xr-xrwt',
    '1160' => '--xrw---T',
    '1161' => '--xrw---t',
    '1162' => '--xrw--wT',
    '1163' => '--xrw--wt',
    '1164' => '--xrw-r-T',
    '1165' => '--xrw-r-t',
    '1166' => '--xrw-rwT',
    '1167' => '--xrw-rwt',
    '1170' => '--xrwx--T',
    '1171' => '--xrwx--t',
    '1172' => '--xrwx-wT',
    '1173' => '--xrwx-wt',
    '1174' => '--xrwxr-T',
    '1175' => '--xrwxr-t',
    '1176' => '--xrwxrwT',
    '1177' => '--xrwxrwt',
    '1200' => '-w------T',
    '1201' => '-w------t',
    '1202' => '-w-----wT',
    '1203' => '-w-----wt',
    '1204' => '-w----r-T',
    '1205' => '-w----r-t',
    '1206' => '-w----rwT',
    '1207' => '-w----rwt',
    '1210' => '-w---x--T',
    '1211' => '-w---x--t',
    '1212' => '-w---x-wT',
    '1213' => '-w---x-wt',
    '1214' => '-w---xr-T',
    '1215' => '-w---xr-t',
    '1216' => '-w---xrwT',
    '1217' => '-w---xrwt',
    '1220' => '-w--w---T',
    '1221' => '-w--w---t',
    '1222' => '-w--w--wT',
    '1223' => '-w--w--wt',
    '1224' => '-w--w-r-T',
    '1225' => '-w--w-r-t',
    '1226' => '-w--w-rwT',
    '1227' => '-w--w-rwt',
    '1230' => '-w--wx--T',
    '1231' => '-w--wx--t',
    '1232' => '-w--wx-wT',
    '1233' => '-w--wx-wt',
    '1234' => '-w--wxr-T',
    '1235' => '-w--wxr-t',
    '1236' => '-w--wxrwT',
    '1237' => '-w--wxrwt',
    '1240' => '-w-r----T',
    '1241' => '-w-r----t',
    '1242' => '-w-r---wT',
    '1243' => '-w-r---wt',
    '1244' => '-w-r--r-T',
    '1245' => '-w-r--r-t',
    '1246' => '-w-r--rwT',
    '1247' => '-w-r--rwt',
    '1250' => '-w-r-x--T',
    '1251' => '-w-r-x--t',
    '1252' => '-w-r-x-wT',
    '1253' => '-w-r-x-wt',
    '1254' => '-w-r-xr-T',
    '1255' => '-w-r-xr-t',
    '1256' => '-w-r-xrwT',
    '1257' => '-w-r-xrwt',
    '1260' => '-w-rw---T',
    '1261' => '-w-rw---t',
    '1262' => '-w-rw--wT',
    '1263' => '-w-rw--wt',
    '1264' => '-w-rw-r-T',
    '1265' => '-w-rw-r-t',
    '1266' => '-w-rw-rwT',
    '1267' => '-w-rw-rwt',
    '1270' => '-w-rwx--T',
    '1271' => '-w-rwx--t',
    '1272' => '-w-rwx-wT',
    '1273' => '-w-rwx-wt',
    '1274' => '-w-rwxr-T',
    '1275' => '-w-rwxr-t',
    '1276' => '-w-rwxrwT',
    '1277' => '-w-rwxrwt',
    '1300' => '-wx-----T',
    '1301' => '-wx-----t',
    '1302' => '-wx----wT',
    '1303' => '-wx----wt',
    '1304' => '-wx---r-T',
    '1305' => '-wx---r-t',
    '1306' => '-wx---rwT',
    '1307' => '-wx---rwt',
    '1310' => '-wx--x--T',
    '1311' => '-wx--x--t',
    '1312' => '-wx--x-wT',
    '1313' => '-wx--x-wt',
    '1314' => '-wx--xr-T',
    '1315' => '-wx--xr-t',
    '1316' => '-wx--xrwT',
    '1317' => '-wx--xrwt',
    '1320' => '-wx-w---T',
    '1321' => '-wx-w---t',
    '1322' => '-wx-w--wT',
    '1323' => '-wx-w--wt',
    '1324' => '-wx-w-r-T',
    '1325' => '-wx-w-r-t',
    '1326' => '-wx-w-rwT',
    '1327' => '-wx-w-rwt',
    '1330' => '-wx-wx--T',
    '1331' => '-wx-wx--t',
    '1332' => '-wx-wx-wT',
    '1333' => '-wx-wx-wt',
    '1334' => '-wx-wxr-T',
    '1335' => '-wx-wxr-t',
    '1336' => '-wx-wxrwT',
    '1337' => '-wx-wxrwt',
    '1340' => '-wxr----T',
    '1341' => '-wxr----t',
    '1342' => '-wxr---wT',
    '1343' => '-wxr---wt',
    '1344' => '-wxr--r-T',
    '1345' => '-wxr--r-t',
    '1346' => '-wxr--rwT',
    '1347' => '-wxr--rwt',
    '1350' => '-wxr-x--T',
    '1351' => '-wxr-x--t',
    '1352' => '-wxr-x-wT',
    '1353' => '-wxr-x-wt',
    '1354' => '-wxr-xr-T',
    '1355' => '-wxr-xr-t',
    '1356' => '-wxr-xrwT',
    '1357' => '-wxr-xrwt',
    '1360' => '-wxrw---T',
    '1361' => '-wxrw---t',
    '1362' => '-wxrw--wT',
    '1363' => '-wxrw--wt',
    '1364' => '-wxrw-r-T',
    '1365' => '-wxrw-r-t',
    '1366' => '-wxrw-rwT',
    '1367' => '-wxrw-rwt',
    '1370' => '-wxrwx--T',
    '1371' => '-wxrwx--t',
    '1372' => '-wxrwx-wT',
    '1373' => '-wxrwx-wt',
    '1374' => '-wxrwxr-T',
    '1375' => '-wxrwxr-t',
    '1376' => '-wxrwxrwT',
    '1377' => '-wxrwxrwt',
    '1400' => 'r-------T',
    '1401' => 'r-------t',
    '1402' => 'r------wT',
    '1403' => 'r------wt',
    '1404' => 'r-----r-T',
    '1405' => 'r-----r-t',
    '1406' => 'r-----rwT',
    '1407' => 'r-----rwt',
    '1410' => 'r----x--T',
    '1411' => 'r----x--t',
    '1412' => 'r----x-wT',
    '1413' => 'r----x-wt',
    '1414' => 'r----xr-T',
    '1415' => 'r----xr-t',
    '1416' => 'r----xrwT',
    '1417' => 'r----xrwt',
    '1420' => 'r---w---T',
    '1421' => 'r---w---t',
    '1422' => 'r---w--wT',
    '1423' => 'r---w--wt',
    '1424' => 'r---w-r-T',
    '1425' => 'r---w-r-t',
    '1426' => 'r---w-rwT',
    '1427' => 'r---w-rwt',
    '1430' => 'r---wx--T',
    '1431' => 'r---wx--t',
    '1432' => 'r---wx-wT',
    '1433' => 'r---wx-wt',
    '1434' => 'r---wxr-T',
    '1435' => 'r---wxr-t',
    '1436' => 'r---wxrwT',
    '1437' => 'r---wxrwt',
    '1440' => 'r--r----T',
    '1441' => 'r--r----t',
    '1442' => 'r--r---wT',
    '1443' => 'r--r---wt',
    '1444' => 'r--r--r-T',
    '1445' => 'r--r--r-t',
    '1446' => 'r--r--rwT',
    '1447' => 'r--r--rwt',
    '1450' => 'r--r-x--T',
    '1451' => 'r--r-x--t',
    '1452' => 'r--r-x-wT',
    '1453' => 'r--r-x-wt',
    '1454' => 'r--r-xr-T',
    '1455' => 'r--r-xr-t',
    '1456' => 'r--r-xrwT',
    '1457' => 'r--r-xrwt',
    '1460' => 'r--rw---T',
    '1461' => 'r--rw---t',
    '1462' => 'r--rw--wT',
    '1463' => 'r--rw--wt',
    '1464' => 'r--rw-r-T',
    '1465' => 'r--rw-r-t',
    '1466' => 'r--rw-rwT',
    '1467' => 'r--rw-rwt',
    '1470' => 'r--rwx--T',
    '1471' => 'r--rwx--t',
    '1472' => 'r--rwx-wT',
    '1473' => 'r--rwx-wt',
    '1474' => 'r--rwxr-T',
    '1475' => 'r--rwxr-t',
    '1476' => 'r--rwxrwT',
    '1477' => 'r--rwxrwt',
    '1500' => 'r-x-----T',
    '1501' => 'r-x-----t',
    '1502' => 'r-x----wT',
    '1503' => 'r-x----wt',
    '1504' => 'r-x---r-T',
    '1505' => 'r-x---r-t',
    '1506' => 'r-x---rwT',
    '1507' => 'r-x---rwt',
    '1510' => 'r-x--x--T',
    '1511' => 'r-x--x--t',
    '1512' => 'r-x--x-wT',
    '1513' => 'r-x--x-wt',
    '1514' => 'r-x--xr-T',
    '1515' => 'r-x--xr-t',
    '1516' => 'r-x--xrwT',
    '1517' => 'r-x--xrwt',
    '1520' => 'r-x-w---T',
    '1521' => 'r-x-w---t',
    '1522' => 'r-x-w--wT',
    '1523' => 'r-x-w--wt',
    '1524' => 'r-x-w-r-T',
    '1525' => 'r-x-w-r-t',
    '1526' => 'r-x-w-rwT',
    '1527' => 'r-x-w-rwt',
    '1530' => 'r-x-wx--T',
    '1531' => 'r-x-wx--t',
    '1532' => 'r-x-wx-wT',
    '1533' => 'r-x-wx-wt',
    '1534' => 'r-x-wxr-T',
    '1535' => 'r-x-wxr-t',
    '1536' => 'r-x-wxrwT',
    '1537' => 'r-x-wxrwt',
    '1540' => 'r-xr----T',
    '1541' => 'r-xr----t',
    '1542' => 'r-xr---wT',
    '1543' => 'r-xr---wt',
    '1544' => 'r-xr--r-T',
    '1545' => 'r-xr--r-t',
    '1546' => 'r-xr--rwT',
    '1547' => 'r-xr--rwt',
    '1550' => 'r-xr-x--T',
    '1551' => 'r-xr-x--t',
    '1552' => 'r-xr-x-wT',
    '1553' => 'r-xr-x-wt',
    '1554' => 'r-xr-xr-T',
    '1555' => 'r-xr-xr-t',
    '1556' => 'r-xr-xrwT',
    '1557' => 'r-xr-xrwt',
    '1560' => 'r-xrw---T',
    '1561' => 'r-xrw---t',
    '1562' => 'r-xrw--wT',
    '1563' => 'r-xrw--wt',
    '1564' => 'r-xrw-r-T',
    '1565' => 'r-xrw-r-t',
    '1566' => 'r-xrw-rwT',
    '1567' => 'r-xrw-rwt',
    '1570' => 'r-xrwx--T',
    '1571' => 'r-xrwx--t',
    '1572' => 'r-xrwx-wT',
    '1573' => 'r-xrwx-wt',
    '1574' => 'r-xrwxr-T',
    '1575' => 'r-xrwxr-t',
    '1576' => 'r-xrwxrwT',
    '1577' => 'r-xrwxrwt',
    '1600' => 'rw------T',
    '1601' => 'rw------t',
    '1602' => 'rw-----wT',
    '1603' => 'rw-----wt',
    '1604' => 'rw----r-T',
    '1605' => 'rw----r-t',
    '1606' => 'rw----rwT',
    '1607' => 'rw----rwt',
    '1610' => 'rw---x--T',
    '1611' => 'rw---x--t',
    '1612' => 'rw---x-wT',
    '1613' => 'rw---x-wt',
    '1614' => 'rw---xr-T',
    '1615' => 'rw---xr-t',
    '1616' => 'rw---xrwT',
    '1617' => 'rw---xrwt',
    '1620' => 'rw--w---T',
    '1621' => 'rw--w---t',
    '1622' => 'rw--w--wT',
    '1623' => 'rw--w--wt',
    '1624' => 'rw--w-r-T',
    '1625' => 'rw--w-r-t',
    '1626' => 'rw--w-rwT',
    '1627' => 'rw--w-rwt',
    '1630' => 'rw--wx--T',
    '1631' => 'rw--wx--t',
    '1632' => 'rw--wx-wT',
    '1633' => 'rw--wx-wt',
    '1634' => 'rw--wxr-T',
    '1635' => 'rw--wxr-t',
    '1636' => 'rw--wxrwT',
    '1637' => 'rw--wxrwt',
    '1640' => 'rw-r----T',
    '1641' => 'rw-r----t',
    '1642' => 'rw-r---wT',
    '1643' => 'rw-r---wt',
    '1644' => 'rw-r--r-T',
    '1645' => 'rw-r--r-t',
    '1646' => 'rw-r--rwT',
    '1647' => 'rw-r--rwt',
    '1650' => 'rw-r-x--T',
    '1651' => 'rw-r-x--t',
    '1652' => 'rw-r-x-wT',
    '1653' => 'rw-r-x-wt',
    '1654' => 'rw-r-xr-T',
    '1655' => 'rw-r-xr-t',
    '1656' => 'rw-r-xrwT',
    '1657' => 'rw-r-xrwt',
    '1660' => 'rw-rw---T',
    '1661' => 'rw-rw---t',
    '1662' => 'rw-rw--wT',
    '1663' => 'rw-rw--wt',
    '1664' => 'rw-rw-r-T',
    '1665' => 'rw-rw-r-t',
    '1666' => 'rw-rw-rwT',
    '1667' => 'rw-rw-rwt',
    '1670' => 'rw-rwx--T',
    '1671' => 'rw-rwx--t',
    '1672' => 'rw-rwx-wT',
    '1673' => 'rw-rwx-wt',
    '1674' => 'rw-rwxr-T',
    '1675' => 'rw-rwxr-t',
    '1676' => 'rw-rwxrwT',
    '1677' => 'rw-rwxrwt',
    '1700' => 'rwx-----T',
    '1701' => 'rwx-----t',
    '1702' => 'rwx----wT',
    '1703' => 'rwx----wt',
    '1704' => 'rwx---r-T',
    '1705' => 'rwx---r-t',
    '1706' => 'rwx---rwT',
    '1707' => 'rwx---rwt',
    '1710' => 'rwx--x--T',
    '1711' => 'rwx--x--t',
    '1712' => 'rwx--x-wT',
    '1713' => 'rwx--x-wt',
    '1714' => 'rwx--xr-T',
    '1715' => 'rwx--xr-t',
    '1716' => 'rwx--xrwT',
    '1717' => 'rwx--xrwt',
    '1720' => 'rwx-w---T',
    '1721' => 'rwx-w---t',
    '1722' => 'rwx-w--wT',
    '1723' => 'rwx-w--wt',
    '1724' => 'rwx-w-r-T',
    '1725' => 'rwx-w-r-t',
    '1726' => 'rwx-w-rwT',
    '1727' => 'rwx-w-rwt',
    '1730' => 'rwx-wx--T',
    '1731' => 'rwx-wx--t',
    '1732' => 'rwx-wx-wT',
    '1733' => 'rwx-wx-wt',
    '1734' => 'rwx-wxr-T',
    '1735' => 'rwx-wxr-t',
    '1736' => 'rwx-wxrwT',
    '1737' => 'rwx-wxrwt',
    '1740' => 'rwxr----T',
    '1741' => 'rwxr----t',
    '1742' => 'rwxr---wT',
    '1743' => 'rwxr---wt',
    '1744' => 'rwxr--r-T',
    '1745' => 'rwxr--r-t',
    '1746' => 'rwxr--rwT',
    '1747' => 'rwxr--rwt',
    '1750' => 'rwxr-x--T',
    '1751' => 'rwxr-x--t',
    '1752' => 'rwxr-x-wT',
    '1753' => 'rwxr-x-wt',
    '1754' => 'rwxr-xr-T',
    '1755' => 'rwxr-xr-t',
    '1756' => 'rwxr-xrwT',
    '1757' => 'rwxr-xrwt',
    '1760' => 'rwxrw---T',
    '1761' => 'rwxrw---t',
    '1762' => 'rwxrw--wT',
    '1763' => 'rwxrw--wt',
    '1764' => 'rwxrw-r-T',
    '1765' => 'rwxrw-r-t',
    '1766' => 'rwxrw-rwT',
    '1767' => 'rwxrw-rwt',
    '1770' => 'rwxrwx--T',
    '1771' => 'rwxrwx--t',
    '1772' => 'rwxrwx-wT',
    '1773' => 'rwxrwx-wt',
    '1774' => 'rwxrwxr-T',
    '1775' => 'rwxrwxr-t',
    '1776' => 'rwxrwxrwT',
    '1777' => 'rwxrwxrwt',
    '2000' => '-----S---',
    '2001' => '-----S--x',
    '2002' => '-----S-w-',
    '2003' => '-----S-wx',
    '2004' => '-----Sr--',
    '2005' => '-----Sr-x',
    '2006' => '-----Srw-',
    '2007' => '-----Srwx',
    '2010' => '-----s---',
    '2011' => '-----s--x',
    '2012' => '-----s-w-',
    '2013' => '-----s-wx',
    '2014' => '-----sr--',
    '2015' => '-----sr-x',
    '2016' => '-----srw-',
    '2017' => '-----srwx',
    '2020' => '----wS---',
    '2021' => '----wS--x',
    '2022' => '----wS-w-',
    '2023' => '----wS-wx',
    '2024' => '----wSr--',
    '2025' => '----wSr-x',
    '2026' => '----wSrw-',
    '2027' => '----wSrwx',
    '2030' => '----ws---',
    '2031' => '----ws--x',
    '2032' => '----ws-w-',
    '2033' => '----ws-wx',
    '2034' => '----wsr--',
    '2035' => '----wsr-x',
    '2036' => '----wsrw-',
    '2037' => '----wsrwx',
    '2040' => '---r-S---',
    '2041' => '---r-S--x',
    '2042' => '---r-S-w-',
    '2043' => '---r-S-wx',
    '2044' => '---r-Sr--',
    '2045' => '---r-Sr-x',
    '2046' => '---r-Srw-',
    '2047' => '---r-Srwx',
    '2050' => '---r-s---',
    '2051' => '---r-s--x',
    '2052' => '---r-s-w-',
    '2053' => '---r-s-wx',
    '2054' => '---r-sr--',
    '2055' => '---r-sr-x',
    '2056' => '---r-srw-',
    '2057' => '---r-srwx',
    '2060' => '---rwS---',
    '2061' => '---rwS--x',
    '2062' => '---rwS-w-',
    '2063' => '---rwS-wx',
    '2064' => '---rwSr--',
    '2065' => '---rwSr-x',
    '2066' => '---rwSrw-',
    '2067' => '---rwSrwx',
    '2070' => '---rws---',
    '2071' => '---rws--x',
    '2072' => '---rws-w-',
    '2073' => '---rws-wx',
    '2074' => '---rwsr--',
    '2075' => '---rwsr-x',
    '2076' => '---rwsrw-',
    '2077' => '---rwsrwx',
    '2100' => '--x--S---',
    '2101' => '--x--S--x',
    '2102' => '--x--S-w-',
    '2103' => '--x--S-wx',
    '2104' => '--x--Sr--',
    '2105' => '--x--Sr-x',
    '2106' => '--x--Srw-',
    '2107' => '--x--Srwx',
    '2110' => '--x--s---',
    '2111' => '--x--s--x',
    '2112' => '--x--s-w-',
    '2113' => '--x--s-wx',
    '2114' => '--x--sr--',
    '2115' => '--x--sr-x',
    '2116' => '--x--srw-',
    '2117' => '--x--srwx',
    '2120' => '--x-wS---',
    '2121' => '--x-wS--x',
    '2122' => '--x-wS-w-',
    '2123' => '--x-wS-wx',
    '2124' => '--x-wSr--',
    '2125' => '--x-wSr-x',
    '2126' => '--x-wSrw-',
    '2127' => '--x-wSrwx',
    '2130' => '--x-ws---',
    '2131' => '--x-ws--x',
    '2132' => '--x-ws-w-',
    '2133' => '--x-ws-wx',
    '2134' => '--x-wsr--',
    '2135' => '--x-wsr-x',
    '2136' => '--x-wsrw-',
    '2137' => '--x-wsrwx',
    '2140' => '--xr-S---',
    '2141' => '--xr-S--x',
    '2142' => '--xr-S-w-',
    '2143' => '--xr-S-wx',
    '2144' => '--xr-Sr--',
    '2145' => '--xr-Sr-x',
    '2146' => '--xr-Srw-',
    '2147' => '--xr-Srwx',
    '2150' => '--xr-s---',
    '2151' => '--xr-s--x',
    '2152' => '--xr-s-w-',
    '2153' => '--xr-s-wx',
    '2154' => '--xr-sr--',
    '2155' => '--xr-sr-x',
    '2156' => '--xr-srw-',
    '2157' => '--xr-srwx',
    '2160' => '--xrwS---',
    '2161' => '--xrwS--x',
    '2162' => '--xrwS-w-',
    '2163' => '--xrwS-wx',
    '2164' => '--xrwSr--',
    '2165' => '--xrwSr-x',
    '2166' => '--xrwSrw-',
    '2167' => '--xrwSrwx',
    '2170' => '--xrws---',
    '2171' => '--xrws--x',
    '2172' => '--xrws-w-',
    '2173' => '--xrws-wx',
    '2174' => '--xrwsr--',
    '2175' => '--xrwsr-x',
    '2176' => '--xrwsrw-',
    '2177' => '--xrwsrwx',
    '2200' => '-w---S---',
    '2201' => '-w---S--x',
    '2202' => '-w---S-w-',
    '2203' => '-w---S-wx',
    '2204' => '-w---Sr--',
    '2205' => '-w---Sr-x',
    '2206' => '-w---Srw-',
    '2207' => '-w---Srwx',
    '2210' => '-w---s---',
    '2211' => '-w---s--x',
    '2212' => '-w---s-w-',
    '2213' => '-w---s-wx',
    '2214' => '-w---sr--',
    '2215' => '-w---sr-x',
    '2216' => '-w---srw-',
    '2217' => '-w---srwx',
    '2220' => '-w--wS---',
    '2221' => '-w--wS--x',
    '2222' => '-w--wS-w-',
    '2223' => '-w--wS-wx',
    '2224' => '-w--wSr--',
    '2225' => '-w--wSr-x',
    '2226' => '-w--wSrw-',
    '2227' => '-w--wSrwx',
    '2230' => '-w--ws---',
    '2231' => '-w--ws--x',
    '2232' => '-w--ws-w-',
    '2233' => '-w--ws-wx',
    '2234' => '-w--wsr--',
    '2235' => '-w--wsr-x',
    '2236' => '-w--wsrw-',
    '2237' => '-w--wsrwx',
    '2240' => '-w-r-S---',
    '2241' => '-w-r-S--x',
    '2242' => '-w-r-S-w-',
    '2243' => '-w-r-S-wx',
    '2244' => '-w-r-Sr--',
    '2245' => '-w-r-Sr-x',
    '2246' => '-w-r-Srw-',
    '2247' => '-w-r-Srwx',
    '2250' => '-w-r-s---',
    '2251' => '-w-r-s--x',
    '2252' => '-w-r-s-w-',
    '2253' => '-w-r-s-wx',
    '2254' => '-w-r-sr--',
    '2255' => '-w-r-sr-x',
    '2256' => '-w-r-srw-',
    '2257' => '-w-r-srwx',
    '2260' => '-w-rwS---',
    '2261' => '-w-rwS--x',
    '2262' => '-w-rwS-w-',
    '2263' => '-w-rwS-wx',
    '2264' => '-w-rwSr--',
    '2265' => '-w-rwSr-x',
    '2266' => '-w-rwSrw-',
    '2267' => '-w-rwSrwx',
    '2270' => '-w-rws---',
    '2271' => '-w-rws--x',
    '2272' => '-w-rws-w-',
    '2273' => '-w-rws-wx',
    '2274' => '-w-rwsr--',
    '2275' => '-w-rwsr-x',
    '2276' => '-w-rwsrw-',
    '2277' => '-w-rwsrwx',
    '2300' => '-wx--S---',
    '2301' => '-wx--S--x',
    '2302' => '-wx--S-w-',
    '2303' => '-wx--S-wx',
    '2304' => '-wx--Sr--',
    '2305' => '-wx--Sr-x',
    '2306' => '-wx--Srw-',
    '2307' => '-wx--Srwx',
    '2310' => '-wx--s---',
    '2311' => '-wx--s--x',
    '2312' => '-wx--s-w-',
    '2313' => '-wx--s-wx',
    '2314' => '-wx--sr--',
    '2315' => '-wx--sr-x',
    '2316' => '-wx--srw-',
    '2317' => '-wx--srwx',
    '2320' => '-wx-wS---',
    '2321' => '-wx-wS--x',
    '2322' => '-wx-wS-w-',
    '2323' => '-wx-wS-wx',
    '2324' => '-wx-wSr--',
    '2325' => '-wx-wSr-x',
    '2326' => '-wx-wSrw-',
    '2327' => '-wx-wSrwx',
    '2330' => '-wx-ws---',
    '2331' => '-wx-ws--x',
    '2332' => '-wx-ws-w-',
    '2333' => '-wx-ws-wx',
    '2334' => '-wx-wsr--',
    '2335' => '-wx-wsr-x',
    '2336' => '-wx-wsrw-',
    '2337' => '-wx-wsrwx',
    '2340' => '-wxr-S---',
    '2341' => '-wxr-S--x',
    '2342' => '-wxr-S-w-',
    '2343' => '-wxr-S-wx',
    '2344' => '-wxr-Sr--',
    '2345' => '-wxr-Sr-x',
    '2346' => '-wxr-Srw-',
    '2347' => '-wxr-Srwx',
    '2350' => '-wxr-s---',
    '2351' => '-wxr-s--x',
    '2352' => '-wxr-s-w-',
    '2353' => '-wxr-s-wx',
    '2354' => '-wxr-sr--',
    '2355' => '-wxr-sr-x',
    '2356' => '-wxr-srw-',
    '2357' => '-wxr-srwx',
    '2360' => '-wxrwS---',
    '2361' => '-wxrwS--x',
    '2362' => '-wxrwS-w-',
    '2363' => '-wxrwS-wx',
    '2364' => '-wxrwSr--',
    '2365' => '-wxrwSr-x',
    '2366' => '-wxrwSrw-',
    '2367' => '-wxrwSrwx',
    '2370' => '-wxrws---',
    '2371' => '-wxrws--x',
    '2372' => '-wxrws-w-',
    '2373' => '-wxrws-wx',
    '2374' => '-wxrwsr--',
    '2375' => '-wxrwsr-x',
    '2376' => '-wxrwsrw-',
    '2377' => '-wxrwsrwx',
    '2400' => 'r----S---',
    '2401' => 'r----S--x',
    '2402' => 'r----S-w-',
    '2403' => 'r----S-wx',
    '2404' => 'r----Sr--',
    '2405' => 'r----Sr-x',
    '2406' => 'r----Srw-',
    '2407' => 'r----Srwx',
    '2410' => 'r----s---',
    '2411' => 'r----s--x',
    '2412' => 'r----s-w-',
    '2413' => 'r----s-wx',
    '2414' => 'r----sr--',
    '2415' => 'r----sr-x',
    '2416' => 'r----srw-',
    '2417' => 'r----srwx',
    '2420' => 'r---wS---',
    '2421' => 'r---wS--x',
    '2422' => 'r---wS-w-',
    '2423' => 'r---wS-wx',
    '2424' => 'r---wSr--',
    '2425' => 'r---wSr-x',
    '2426' => 'r---wSrw-',
    '2427' => 'r---wSrwx',
    '2430' => 'r---ws---',
    '2431' => 'r---ws--x',
    '2432' => 'r---ws-w-',
    '2433' => 'r---ws-wx',
    '2434' => 'r---wsr--',
    '2435' => 'r---wsr-x',
    '2436' => 'r---wsrw-',
    '2437' => 'r---wsrwx',
    '2440' => 'r--r-S---',
    '2441' => 'r--r-S--x',
    '2442' => 'r--r-S-w-',
    '2443' => 'r--r-S-wx',
    '2444' => 'r--r-Sr--',
    '2445' => 'r--r-Sr-x',
    '2446' => 'r--r-Srw-',
    '2447' => 'r--r-Srwx',
    '2450' => 'r--r-s---',
    '2451' => 'r--r-s--x',
    '2452' => 'r--r-s-w-',
    '2453' => 'r--r-s-wx',
    '2454' => 'r--r-sr--',
    '2455' => 'r--r-sr-x',
    '2456' => 'r--r-srw-',
    '2457' => 'r--r-srwx',
    '2460' => 'r--rwS---',
    '2461' => 'r--rwS--x',
    '2462' => 'r--rwS-w-',
    '2463' => 'r--rwS-wx',
    '2464' => 'r--rwSr--',
    '2465' => 'r--rwSr-x',
    '2466' => 'r--rwSrw-',
    '2467' => 'r--rwSrwx',
    '2470' => 'r--rws---',
    '2471' => 'r--rws--x',
    '2472' => 'r--rws-w-',
    '2473' => 'r--rws-wx',
    '2474' => 'r--rwsr--',
    '2475' => 'r--rwsr-x',
    '2476' => 'r--rwsrw-',
    '2477' => 'r--rwsrwx',
    '2500' => 'r-x--S---',
    '2501' => 'r-x--S--x',
    '2502' => 'r-x--S-w-',
    '2503' => 'r-x--S-wx',
    '2504' => 'r-x--Sr--',
    '2505' => 'r-x--Sr-x',
    '2506' => 'r-x--Srw-',
    '2507' => 'r-x--Srwx',
    '2510' => 'r-x--s---',
    '2511' => 'r-x--s--x',
    '2512' => 'r-x--s-w-',
    '2513' => 'r-x--s-wx',
    '2514' => 'r-x--sr--',
    '2515' => 'r-x--sr-x',
    '2516' => 'r-x--srw-',
    '2517' => 'r-x--srwx',
    '2520' => 'r-x-wS---',
    '2521' => 'r-x-wS--x',
    '2522' => 'r-x-wS-w-',
    '2523' => 'r-x-wS-wx',
    '2524' => 'r-x-wSr--',
    '2525' => 'r-x-wSr-x',
    '2526' => 'r-x-wSrw-',
    '2527' => 'r-x-wSrwx',
    '2530' => 'r-x-ws---',
    '2531' => 'r-x-ws--x',
    '2532' => 'r-x-ws-w-',
    '2533' => 'r-x-ws-wx',
    '2534' => 'r-x-wsr--',
    '2535' => 'r-x-wsr-x',
    '2536' => 'r-x-wsrw-',
    '2537' => 'r-x-wsrwx',
    '2540' => 'r-xr-S---',
    '2541' => 'r-xr-S--x',
    '2542' => 'r-xr-S-w-',
    '2543' => 'r-xr-S-wx',
    '2544' => 'r-xr-Sr--',
    '2545' => 'r-xr-Sr-x',
    '2546' => 'r-xr-Srw-',
    '2547' => 'r-xr-Srwx',
    '2550' => 'r-xr-s---',
    '2551' => 'r-xr-s--x',
    '2552' => 'r-xr-s-w-',
    '2553' => 'r-xr-s-wx',
    '2554' => 'r-xr-sr--',
    '2555' => 'r-xr-sr-x',
    '2556' => 'r-xr-srw-',
    '2557' => 'r-xr-srwx',
    '2560' => 'r-xrwS---',
    '2561' => 'r-xrwS--x',
    '2562' => 'r-xrwS-w-',
    '2563' => 'r-xrwS-wx',
    '2564' => 'r-xrwSr--',
    '2565' => 'r-xrwSr-x',
    '2566' => 'r-xrwSrw-',
    '2567' => 'r-xrwSrwx',
    '2570' => 'r-xrws---',
    '2571' => 'r-xrws--x',
    '2572' => 'r-xrws-w-',
    '2573' => 'r-xrws-wx',
    '2574' => 'r-xrwsr--',
    '2575' => 'r-xrwsr-x',
    '2576' => 'r-xrwsrw-',
    '2577' => 'r-xrwsrwx',
    '2600' => 'rw---S---',
    '2601' => 'rw---S--x',
    '2602' => 'rw---S-w-',
    '2603' => 'rw---S-wx',
    '2604' => 'rw---Sr--',
    '2605' => 'rw---Sr-x',
    '2606' => 'rw---Srw-',
    '2607' => 'rw---Srwx',
    '2610' => 'rw---s---',
    '2611' => 'rw---s--x',
    '2612' => 'rw---s-w-',
    '2613' => 'rw---s-wx',
    '2614' => 'rw---sr--',
    '2615' => 'rw---sr-x',
    '2616' => 'rw---srw-',
    '2617' => 'rw---srwx',
    '2620' => 'rw--wS---',
    '2621' => 'rw--wS--x',
    '2622' => 'rw--wS-w-',
    '2623' => 'rw--wS-wx',
    '2624' => 'rw--wSr--',
    '2625' => 'rw--wSr-x',
    '2626' => 'rw--wSrw-',
    '2627' => 'rw--wSrwx',
    '2630' => 'rw--ws---',
    '2631' => 'rw--ws--x',
    '2632' => 'rw--ws-w-',
    '2633' => 'rw--ws-wx',
    '2634' => 'rw--wsr--',
    '2635' => 'rw--wsr-x',
    '2636' => 'rw--wsrw-',
    '2637' => 'rw--wsrwx',
    '2640' => 'rw-r-S---',
    '2641' => 'rw-r-S--x',
    '2642' => 'rw-r-S-w-',
    '2643' => 'rw-r-S-wx',
    '2644' => 'rw-r-Sr--',
    '2645' => 'rw-r-Sr-x',
    '2646' => 'rw-r-Srw-',
    '2647' => 'rw-r-Srwx',
    '2650' => 'rw-r-s---',
    '2651' => 'rw-r-s--x',
    '2652' => 'rw-r-s-w-',
    '2653' => 'rw-r-s-wx',
    '2654' => 'rw-r-sr--',
    '2655' => 'rw-r-sr-x',
    '2656' => 'rw-r-srw-',
    '2657' => 'rw-r-srwx',
    '2660' => 'rw-rwS---',
    '2661' => 'rw-rwS--x',
    '2662' => 'rw-rwS-w-',
    '2663' => 'rw-rwS-wx',
    '2664' => 'rw-rwSr--',
    '2665' => 'rw-rwSr-x',
    '2666' => 'rw-rwSrw-',
    '2667' => 'rw-rwSrwx',
    '2670' => 'rw-rws---',
    '2671' => 'rw-rws--x',
    '2672' => 'rw-rws-w-',
    '2673' => 'rw-rws-wx',
    '2674' => 'rw-rwsr--',
    '2675' => 'rw-rwsr-x',
    '2676' => 'rw-rwsrw-',
    '2677' => 'rw-rwsrwx',
    '2700' => 'rwx--S---',
    '2701' => 'rwx--S--x',
    '2702' => 'rwx--S-w-',
    '2703' => 'rwx--S-wx',
    '2704' => 'rwx--Sr--',
    '2705' => 'rwx--Sr-x',
    '2706' => 'rwx--Srw-',
    '2707' => 'rwx--Srwx',
    '2710' => 'rwx--s---',
    '2711' => 'rwx--s--x',
    '2712' => 'rwx--s-w-',
    '2713' => 'rwx--s-wx',
    '2714' => 'rwx--sr--',
    '2715' => 'rwx--sr-x',
    '2716' => 'rwx--srw-',
    '2717' => 'rwx--srwx',
    '2720' => 'rwx-wS---',
    '2721' => 'rwx-wS--x',
    '2722' => 'rwx-wS-w-',
    '2723' => 'rwx-wS-wx',
    '2724' => 'rwx-wSr--',
    '2725' => 'rwx-wSr-x',
    '2726' => 'rwx-wSrw-',
    '2727' => 'rwx-wSrwx',
    '2730' => 'rwx-ws---',
    '2731' => 'rwx-ws--x',
    '2732' => 'rwx-ws-w-',
    '2733' => 'rwx-ws-wx',
    '2734' => 'rwx-wsr--',
    '2735' => 'rwx-wsr-x',
    '2736' => 'rwx-wsrw-',
    '2737' => 'rwx-wsrwx',
    '2740' => 'rwxr-S---',
    '2741' => 'rwxr-S--x',
    '2742' => 'rwxr-S-w-',
    '2743' => 'rwxr-S-wx',
    '2744' => 'rwxr-Sr--',
    '2745' => 'rwxr-Sr-x',
    '2746' => 'rwxr-Srw-',
    '2747' => 'rwxr-Srwx',
    '2750' => 'rwxr-s---',
    '2751' => 'rwxr-s--x',
    '2752' => 'rwxr-s-w-',
    '2753' => 'rwxr-s-wx',
    '2754' => 'rwxr-sr--',
    '2755' => 'rwxr-sr-x',
    '2756' => 'rwxr-srw-',
    '2757' => 'rwxr-srwx',
    '2760' => 'rwxrwS---',
    '2761' => 'rwxrwS--x',
    '2762' => 'rwxrwS-w-',
    '2763' => 'rwxrwS-wx',
    '2764' => 'rwxrwSr--',
    '2765' => 'rwxrwSr-x',
    '2766' => 'rwxrwSrw-',
    '2767' => 'rwxrwSrwx',
    '2770' => 'rwxrws---',
    '2771' => 'rwxrws--x',
    '2772' => 'rwxrws-w-',
    '2773' => 'rwxrws-wx',
    '2774' => 'rwxrwsr--',
    '2775' => 'rwxrwsr-x',
    '2776' => 'rwxrwsrw-',
    '2777' => 'rwxrwsrwx',
    '3000' => '-----S--T',
    '3001' => '-----S--t',
    '3002' => '-----S-wT',
    '3003' => '-----S-wt',
    '3004' => '-----Sr-T',
    '3005' => '-----Sr-t',
    '3006' => '-----SrwT',
    '3007' => '-----Srwt',
    '3010' => '-----s--T',
    '3011' => '-----s--t',
    '3012' => '-----s-wT',
    '3013' => '-----s-wt',
    '3014' => '-----sr-T',
    '3015' => '-----sr-t',
    '3016' => '-----srwT',
    '3017' => '-----srwt',
    '3020' => '----wS--T',
    '3021' => '----wS--t',
    '3022' => '----wS-wT',
    '3023' => '----wS-wt',
    '3024' => '----wSr-T',
    '3025' => '----wSr-t',
    '3026' => '----wSrwT',
    '3027' => '----wSrwt',
    '3030' => '----ws--T',
    '3031' => '----ws--t',
    '3032' => '----ws-wT',
    '3033' => '----ws-wt',
    '3034' => '----wsr-T',
    '3035' => '----wsr-t',
    '3036' => '----wsrwT',
    '3037' => '----wsrwt',
    '3040' => '---r-S--T',
    '3041' => '---r-S--t',
    '3042' => '---r-S-wT',
    '3043' => '---r-S-wt',
    '3044' => '---r-Sr-T',
    '3045' => '---r-Sr-t',
    '3046' => '---r-SrwT',
    '3047' => '---r-Srwt',
    '3050' => '---r-s--T',
    '3051' => '---r-s--t',
    '3052' => '---r-s-wT',
    '3053' => '---r-s-wt',
    '3054' => '---r-sr-T',
    '3055' => '---r-sr-t',
    '3056' => '---r-srwT',
    '3057' => '---r-srwt',
    '3060' => '---rwS--T',
    '3061' => '---rwS--t',
    '3062' => '---rwS-wT',
    '3063' => '---rwS-wt',
    '3064' => '---rwSr-T',
    '3065' => '---rwSr-t',
    '3066' => '---rwSrwT',
    '3067' => '---rwSrwt',
    '3070' => '---rws--T',
    '3071' => '---rws--t',
    '3072' => '---rws-wT',
    '3073' => '---rws-wt',
    '3074' => '---rwsr-T',
    '3075' => '---rwsr-t',
    '3076' => '---rwsrwT',
    '3077' => '---rwsrwt',
    '3100' => '--x--S--T',
    '3101' => '--x--S--t',
    '3102' => '--x--S-wT',
    '3103' => '--x--S-wt',
    '3104' => '--x--Sr-T',
    '3105' => '--x--Sr-t',
    '3106' => '--x--SrwT',
    '3107' => '--x--Srwt',
    '3110' => '--x--s--T',
    '3111' => '--x--s--t',
    '3112' => '--x--s-wT',
    '3113' => '--x--s-wt',
    '3114' => '--x--sr-T',
    '3115' => '--x--sr-t',
    '3116' => '--x--srwT',
    '3117' => '--x--srwt',
    '3120' => '--x-wS--T',
    '3121' => '--x-wS--t',
    '3122' => '--x-wS-wT',
    '3123' => '--x-wS-wt',
    '3124' => '--x-wSr-T',
    '3125' => '--x-wSr-t',
    '3126' => '--x-wSrwT',
    '3127' => '--x-wSrwt',
    '3130' => '--x-ws--T',
    '3131' => '--x-ws--t',
    '3132' => '--x-ws-wT',
    '3133' => '--x-ws-wt',
    '3134' => '--x-wsr-T',
    '3135' => '--x-wsr-t',
    '3136' => '--x-wsrwT',
    '3137' => '--x-wsrwt',
    '3140' => '--xr-S--T',
    '3141' => '--xr-S--t',
    '3142' => '--xr-S-wT',
    '3143' => '--xr-S-wt',
    '3144' => '--xr-Sr-T',
    '3145' => '--xr-Sr-t',
    '3146' => '--xr-SrwT',
    '3147' => '--xr-Srwt',
    '3150' => '--xr-s--T',
    '3151' => '--xr-s--t',
    '3152' => '--xr-s-wT',
    '3153' => '--xr-s-wt',
    '3154' => '--xr-sr-T',
    '3155' => '--xr-sr-t',
    '3156' => '--xr-srwT',
    '3157' => '--xr-srwt',
    '3160' => '--xrwS--T',
    '3161' => '--xrwS--t',
    '3162' => '--xrwS-wT',
    '3163' => '--xrwS-wt',
    '3164' => '--xrwSr-T',
    '3165' => '--xrwSr-t',
    '3166' => '--xrwSrwT',
    '3167' => '--xrwSrwt',
    '3170' => '--xrws--T',
    '3171' => '--xrws--t',
    '3172' => '--xrws-wT',
    '3173' => '--xrws-wt',
    '3174' => '--xrwsr-T',
    '3175' => '--xrwsr-t',
    '3176' => '--xrwsrwT',
    '3177' => '--xrwsrwt',
    '3200' => '-w---S--T',
    '3201' => '-w---S--t',
    '3202' => '-w---S-wT',
    '3203' => '-w---S-wt',
    '3204' => '-w---Sr-T',
    '3205' => '-w---Sr-t',
    '3206' => '-w---SrwT',
    '3207' => '-w---Srwt',
    '3210' => '-w---s--T',
    '3211' => '-w---s--t',
    '3212' => '-w---s-wT',
    '3213' => '-w---s-wt',
    '3214' => '-w---sr-T',
    '3215' => '-w---sr-t',
    '3216' => '-w---srwT',
    '3217' => '-w---srwt',
    '3220' => '-w--wS--T',
    '3221' => '-w--wS--t',
    '3222' => '-w--wS-wT',
    '3223' => '-w--wS-wt',
    '3224' => '-w--wSr-T',
    '3225' => '-w--wSr-t',
    '3226' => '-w--wSrwT',
    '3227' => '-w--wSrwt',
    '3230' => '-w--ws--T',
    '3231' => '-w--ws--t',
    '3232' => '-w--ws-wT',
    '3233' => '-w--ws-wt',
    '3234' => '-w--wsr-T',
    '3235' => '-w--wsr-t',
    '3236' => '-w--wsrwT',
    '3237' => '-w--wsrwt',
    '3240' => '-w-r-S--T',
    '3241' => '-w-r-S--t',
    '3242' => '-w-r-S-wT',
    '3243' => '-w-r-S-wt',
    '3244' => '-w-r-Sr-T',
    '3245' => '-w-r-Sr-t',
    '3246' => '-w-r-SrwT',
    '3247' => '-w-r-Srwt',
    '3250' => '-w-r-s--T',
    '3251' => '-w-r-s--t',
    '3252' => '-w-r-s-wT',
    '3253' => '-w-r-s-wt',
    '3254' => '-w-r-sr-T',
    '3255' => '-w-r-sr-t',
    '3256' => '-w-r-srwT',
    '3257' => '-w-r-srwt',
    '3260' => '-w-rwS--T',
    '3261' => '-w-rwS--t',
    '3262' => '-w-rwS-wT',
    '3263' => '-w-rwS-wt',
    '3264' => '-w-rwSr-T',
    '3265' => '-w-rwSr-t',
    '3266' => '-w-rwSrwT',
    '3267' => '-w-rwSrwt',
    '3270' => '-w-rws--T',
    '3271' => '-w-rws--t',
    '3272' => '-w-rws-wT',
    '3273' => '-w-rws-wt',
    '3274' => '-w-rwsr-T',
    '3275' => '-w-rwsr-t',
    '3276' => '-w-rwsrwT',
    '3277' => '-w-rwsrwt',
    '3300' => '-wx--S--T',
    '3301' => '-wx--S--t',
    '3302' => '-wx--S-wT',
    '3303' => '-wx--S-wt',
    '3304' => '-wx--Sr-T',
    '3305' => '-wx--Sr-t',
    '3306' => '-wx--SrwT',
    '3307' => '-wx--Srwt',
    '3310' => '-wx--s--T',
    '3311' => '-wx--s--t',
    '3312' => '-wx--s-wT',
    '3313' => '-wx--s-wt',
    '3314' => '-wx--sr-T',
    '3315' => '-wx--sr-t',
    '3316' => '-wx--srwT',
    '3317' => '-wx--srwt',
    '3320' => '-wx-wS--T',
    '3321' => '-wx-wS--t',
    '3322' => '-wx-wS-wT',
    '3323' => '-wx-wS-wt',
    '3324' => '-wx-wSr-T',
    '3325' => '-wx-wSr-t',
    '3326' => '-wx-wSrwT',
    '3327' => '-wx-wSrwt',
    '3330' => '-wx-ws--T',
    '3331' => '-wx-ws--t',
    '3332' => '-wx-ws-wT',
    '3333' => '-wx-ws-wt',
    '3334' => '-wx-wsr-T',
    '3335' => '-wx-wsr-t',
    '3336' => '-wx-wsrwT',
    '3337' => '-wx-wsrwt',
    '3340' => '-wxr-S--T',
    '3341' => '-wxr-S--t',
    '3342' => '-wxr-S-wT',
    '3343' => '-wxr-S-wt',
    '3344' => '-wxr-Sr-T',
    '3345' => '-wxr-Sr-t',
    '3346' => '-wxr-SrwT',
    '3347' => '-wxr-Srwt',
    '3350' => '-wxr-s--T',
    '3351' => '-wxr-s--t',
    '3352' => '-wxr-s-wT',
    '3353' => '-wxr-s-wt',
    '3354' => '-wxr-sr-T',
    '3355' => '-wxr-sr-t',
    '3356' => '-wxr-srwT',
    '3357' => '-wxr-srwt',
    '3360' => '-wxrwS--T',
    '3361' => '-wxrwS--t',
    '3362' => '-wxrwS-wT',
    '3363' => '-wxrwS-wt',
    '3364' => '-wxrwSr-T',
    '3365' => '-wxrwSr-t',
    '3366' => '-wxrwSrwT',
    '3367' => '-wxrwSrwt',
    '3370' => '-wxrws--T',
    '3371' => '-wxrws--t',
    '3372' => '-wxrws-wT',
    '3373' => '-wxrws-wt',
    '3374' => '-wxrwsr-T',
    '3375' => '-wxrwsr-t',
    '3376' => '-wxrwsrwT',
    '3377' => '-wxrwsrwt',
    '3400' => 'r----S--T',
    '3401' => 'r----S--t',
    '3402' => 'r----S-wT',
    '3403' => 'r----S-wt',
    '3404' => 'r----Sr-T',
    '3405' => 'r----Sr-t',
    '3406' => 'r----SrwT',
    '3407' => 'r----Srwt',
    '3410' => 'r----s--T',
    '3411' => 'r----s--t',
    '3412' => 'r----s-wT',
    '3413' => 'r----s-wt',
    '3414' => 'r----sr-T',
    '3415' => 'r----sr-t',
    '3416' => 'r----srwT',
    '3417' => 'r----srwt',
    '3420' => 'r---wS--T',
    '3421' => 'r---wS--t',
    '3422' => 'r---wS-wT',
    '3423' => 'r---wS-wt',
    '3424' => 'r---wSr-T',
    '3425' => 'r---wSr-t',
    '3426' => 'r---wSrwT',
    '3427' => 'r---wSrwt',
    '3430' => 'r---ws--T',
    '3431' => 'r---ws--t',
    '3432' => 'r---ws-wT',
    '3433' => 'r---ws-wt',
    '3434' => 'r---wsr-T',
    '3435' => 'r---wsr-t',
    '3436' => 'r---wsrwT',
    '3437' => 'r---wsrwt',
    '3440' => 'r--r-S--T',
    '3441' => 'r--r-S--t',
    '3442' => 'r--r-S-wT',
    '3443' => 'r--r-S-wt',
    '3444' => 'r--r-Sr-T',
    '3445' => 'r--r-Sr-t',
    '3446' => 'r--r-SrwT',
    '3447' => 'r--r-Srwt',
    '3450' => 'r--r-s--T',
    '3451' => 'r--r-s--t',
    '3452' => 'r--r-s-wT',
    '3453' => 'r--r-s-wt',
    '3454' => 'r--r-sr-T',
    '3455' => 'r--r-sr-t',
    '3456' => 'r--r-srwT',
    '3457' => 'r--r-srwt',
    '3460' => 'r--rwS--T',
    '3461' => 'r--rwS--t',
    '3462' => 'r--rwS-wT',
    '3463' => 'r--rwS-wt',
    '3464' => 'r--rwSr-T',
    '3465' => 'r--rwSr-t',
    '3466' => 'r--rwSrwT',
    '3467' => 'r--rwSrwt',
    '3470' => 'r--rws--T',
    '3471' => 'r--rws--t',
    '3472' => 'r--rws-wT',
    '3473' => 'r--rws-wt',
    '3474' => 'r--rwsr-T',
    '3475' => 'r--rwsr-t',
    '3476' => 'r--rwsrwT',
    '3477' => 'r--rwsrwt',
    '3500' => 'r-x--S--T',
    '3501' => 'r-x--S--t',
    '3502' => 'r-x--S-wT',
    '3503' => 'r-x--S-wt',
    '3504' => 'r-x--Sr-T',
    '3505' => 'r-x--Sr-t',
    '3506' => 'r-x--SrwT',
    '3507' => 'r-x--Srwt',
    '3510' => 'r-x--s--T',
    '3511' => 'r-x--s--t',
    '3512' => 'r-x--s-wT',
    '3513' => 'r-x--s-wt',
    '3514' => 'r-x--sr-T',
    '3515' => 'r-x--sr-t',
    '3516' => 'r-x--srwT',
    '3517' => 'r-x--srwt',
    '3520' => 'r-x-wS--T',
    '3521' => 'r-x-wS--t',
    '3522' => 'r-x-wS-wT',
    '3523' => 'r-x-wS-wt',
    '3524' => 'r-x-wSr-T',
    '3525' => 'r-x-wSr-t',
    '3526' => 'r-x-wSrwT',
    '3527' => 'r-x-wSrwt',
    '3530' => 'r-x-ws--T',
    '3531' => 'r-x-ws--t',
    '3532' => 'r-x-ws-wT',
    '3533' => 'r-x-ws-wt',
    '3534' => 'r-x-wsr-T',
    '3535' => 'r-x-wsr-t',
    '3536' => 'r-x-wsrwT',
    '3537' => 'r-x-wsrwt',
    '3540' => 'r-xr-S--T',
    '3541' => 'r-xr-S--t',
    '3542' => 'r-xr-S-wT',
    '3543' => 'r-xr-S-wt',
    '3544' => 'r-xr-Sr-T',
    '3545' => 'r-xr-Sr-t',
    '3546' => 'r-xr-SrwT',
    '3547' => 'r-xr-Srwt',
    '3550' => 'r-xr-s--T',
    '3551' => 'r-xr-s--t',
    '3552' => 'r-xr-s-wT',
    '3553' => 'r-xr-s-wt',
    '3554' => 'r-xr-sr-T',
    '3555' => 'r-xr-sr-t',
    '3556' => 'r-xr-srwT',
    '3557' => 'r-xr-srwt',
    '3560' => 'r-xrwS--T',
    '3561' => 'r-xrwS--t',
    '3562' => 'r-xrwS-wT',
    '3563' => 'r-xrwS-wt',
    '3564' => 'r-xrwSr-T',
    '3565' => 'r-xrwSr-t',
    '3566' => 'r-xrwSrwT',
    '3567' => 'r-xrwSrwt',
    '3570' => 'r-xrws--T',
    '3571' => 'r-xrws--t',
    '3572' => 'r-xrws-wT',
    '3573' => 'r-xrws-wt',
    '3574' => 'r-xrwsr-T',
    '3575' => 'r-xrwsr-t',
    '3576' => 'r-xrwsrwT',
    '3577' => 'r-xrwsrwt',
    '3600' => 'rw---S--T',
    '3601' => 'rw---S--t',
    '3602' => 'rw---S-wT',
    '3603' => 'rw---S-wt',
    '3604' => 'rw---Sr-T',
    '3605' => 'rw---Sr-t',
    '3606' => 'rw---SrwT',
    '3607' => 'rw---Srwt',
    '3610' => 'rw---s--T',
    '3611' => 'rw---s--t',
    '3612' => 'rw---s-wT',
    '3613' => 'rw---s-wt',
    '3614' => 'rw---sr-T',
    '3615' => 'rw---sr-t',
    '3616' => 'rw---srwT',
    '3617' => 'rw---srwt',
    '3620' => 'rw--wS--T',
    '3621' => 'rw--wS--t',
    '3622' => 'rw--wS-wT',
    '3623' => 'rw--wS-wt',
    '3624' => 'rw--wSr-T',
    '3625' => 'rw--wSr-t',
    '3626' => 'rw--wSrwT',
    '3627' => 'rw--wSrwt',
    '3630' => 'rw--ws--T',
    '3631' => 'rw--ws--t',
    '3632' => 'rw--ws-wT',
    '3633' => 'rw--ws-wt',
    '3634' => 'rw--wsr-T',
    '3635' => 'rw--wsr-t',
    '3636' => 'rw--wsrwT',
    '3637' => 'rw--wsrwt',
    '3640' => 'rw-r-S--T',
    '3641' => 'rw-r-S--t',
    '3642' => 'rw-r-S-wT',
    '3643' => 'rw-r-S-wt',
    '3644' => 'rw-r-Sr-T',
    '3645' => 'rw-r-Sr-t',
    '3646' => 'rw-r-SrwT',
    '3647' => 'rw-r-Srwt',
    '3650' => 'rw-r-s--T',
    '3651' => 'rw-r-s--t',
    '3652' => 'rw-r-s-wT',
    '3653' => 'rw-r-s-wt',
    '3654' => 'rw-r-sr-T',
    '3655' => 'rw-r-sr-t',
    '3656' => 'rw-r-srwT',
    '3657' => 'rw-r-srwt',
    '3660' => 'rw-rwS--T',
    '3661' => 'rw-rwS--t',
    '3662' => 'rw-rwS-wT',
    '3663' => 'rw-rwS-wt',
    '3664' => 'rw-rwSr-T',
    '3665' => 'rw-rwSr-t',
    '3666' => 'rw-rwSrwT',
    '3667' => 'rw-rwSrwt',
    '3670' => 'rw-rws--T',
    '3671' => 'rw-rws--t',
    '3672' => 'rw-rws-wT',
    '3673' => 'rw-rws-wt',
    '3674' => 'rw-rwsr-T',
    '3675' => 'rw-rwsr-t',
    '3676' => 'rw-rwsrwT',
    '3677' => 'rw-rwsrwt',
    '3700' => 'rwx--S--T',
    '3701' => 'rwx--S--t',
    '3702' => 'rwx--S-wT',
    '3703' => 'rwx--S-wt',
    '3704' => 'rwx--Sr-T',
    '3705' => 'rwx--Sr-t',
    '3706' => 'rwx--SrwT',
    '3707' => 'rwx--Srwt',
    '3710' => 'rwx--s--T',
    '3711' => 'rwx--s--t',
    '3712' => 'rwx--s-wT',
    '3713' => 'rwx--s-wt',
    '3714' => 'rwx--sr-T',
    '3715' => 'rwx--sr-t',
    '3716' => 'rwx--srwT',
    '3717' => 'rwx--srwt',
    '3720' => 'rwx-wS--T',
    '3721' => 'rwx-wS--t',
    '3722' => 'rwx-wS-wT',
    '3723' => 'rwx-wS-wt',
    '3724' => 'rwx-wSr-T',
    '3725' => 'rwx-wSr-t',
    '3726' => 'rwx-wSrwT',
    '3727' => 'rwx-wSrwt',
    '3730' => 'rwx-ws--T',
    '3731' => 'rwx-ws--t',
    '3732' => 'rwx-ws-wT',
    '3733' => 'rwx-ws-wt',
    '3734' => 'rwx-wsr-T',
    '3735' => 'rwx-wsr-t',
    '3736' => 'rwx-wsrwT',
    '3737' => 'rwx-wsrwt',
    '3740' => 'rwxr-S--T',
    '3741' => 'rwxr-S--t',
    '3742' => 'rwxr-S-wT',
    '3743' => 'rwxr-S-wt',
    '3744' => 'rwxr-Sr-T',
    '3745' => 'rwxr-Sr-t',
    '3746' => 'rwxr-SrwT',
    '3747' => 'rwxr-Srwt',
    '3750' => 'rwxr-s--T',
    '3751' => 'rwxr-s--t',
    '3752' => 'rwxr-s-wT',
    '3753' => 'rwxr-s-wt',
    '3754' => 'rwxr-sr-T',
    '3755' => 'rwxr-sr-t',
    '3756' => 'rwxr-srwT',
    '3757' => 'rwxr-srwt',
    '3760' => 'rwxrwS--T',
    '3761' => 'rwxrwS--t',
    '3762' => 'rwxrwS-wT',
    '3763' => 'rwxrwS-wt',
    '3764' => 'rwxrwSr-T',
    '3765' => 'rwxrwSr-t',
    '3766' => 'rwxrwSrwT',
    '3767' => 'rwxrwSrwt',
    '3770' => 'rwxrws--T',
    '3771' => 'rwxrws--t',
    '3772' => 'rwxrws-wT',
    '3773' => 'rwxrws-wt',
    '3774' => 'rwxrwsr-T',
    '3775' => 'rwxrwsr-t',
    '3776' => 'rwxrwsrwT',
    '3777' => 'rwxrwsrwt',
    '4000' => '--S------',
    '4001' => '--S-----x',
    '4002' => '--S----w-',
    '4003' => '--S----wx',
    '4004' => '--S---r--',
    '4005' => '--S---r-x',
    '4006' => '--S---rw-',
    '4007' => '--S---rwx',
    '4010' => '--S--x---',
    '4011' => '--S--x--x',
    '4012' => '--S--x-w-',
    '4013' => '--S--x-wx',
    '4014' => '--S--xr--',
    '4015' => '--S--xr-x',
    '4016' => '--S--xrw-',
    '4017' => '--S--xrwx',
    '4020' => '--S-w----',
    '4021' => '--S-w---x',
    '4022' => '--S-w--w-',
    '4023' => '--S-w--wx',
    '4024' => '--S-w-r--',
    '4025' => '--S-w-r-x',
    '4026' => '--S-w-rw-',
    '4027' => '--S-w-rwx',
    '4030' => '--S-wx---',
    '4031' => '--S-wx--x',
    '4032' => '--S-wx-w-',
    '4033' => '--S-wx-wx',
    '4034' => '--S-wxr--',
    '4035' => '--S-wxr-x',
    '4036' => '--S-wxrw-',
    '4037' => '--S-wxrwx',
    '4040' => '--Sr-----',
    '4041' => '--Sr----x',
    '4042' => '--Sr---w-',
    '4043' => '--Sr---wx',
    '4044' => '--Sr--r--',
    '4045' => '--Sr--r-x',
    '4046' => '--Sr--rw-',
    '4047' => '--Sr--rwx',
    '4050' => '--Sr-x---',
    '4051' => '--Sr-x--x',
    '4052' => '--Sr-x-w-',
    '4053' => '--Sr-x-wx',
    '4054' => '--Sr-xr--',
    '4055' => '--Sr-xr-x',
    '4056' => '--Sr-xrw-',
    '4057' => '--Sr-xrwx',
    '4060' => '--Srw----',
    '4061' => '--Srw---x',
    '4062' => '--Srw--w-',
    '4063' => '--Srw--wx',
    '4064' => '--Srw-r--',
    '4065' => '--Srw-r-x',
    '4066' => '--Srw-rw-',
    '4067' => '--Srw-rwx',
    '4070' => '--Srwx---',
    '4071' => '--Srwx--x',
    '4072' => '--Srwx-w-',
    '4073' => '--Srwx-wx',
    '4074' => '--Srwxr--',
    '4075' => '--Srwxr-x',
    '4076' => '--Srwxrw-',
    '4077' => '--Srwxrwx',
    '4100' => '--s------',
    '4101' => '--s-----x',
    '4102' => '--s----w-',
    '4103' => '--s----wx',
    '4104' => '--s---r--',
    '4105' => '--s---r-x',
    '4106' => '--s---rw-',
    '4107' => '--s---rwx',
    '4110' => '--s--x---',
    '4111' => '--s--x--x',
    '4112' => '--s--x-w-',
    '4113' => '--s--x-wx',
    '4114' => '--s--xr--',
    '4115' => '--s--xr-x',
    '4116' => '--s--xrw-',
    '4117' => '--s--xrwx',
    '4120' => '--s-w----',
    '4121' => '--s-w---x',
    '4122' => '--s-w--w-',
    '4123' => '--s-w--wx',
    '4124' => '--s-w-r--',
    '4125' => '--s-w-r-x',
    '4126' => '--s-w-rw-',
    '4127' => '--s-w-rwx',
    '4130' => '--s-wx---',
    '4131' => '--s-wx--x',
    '4132' => '--s-wx-w-',
    '4133' => '--s-wx-wx',
    '4134' => '--s-wxr--',
    '4135' => '--s-wxr-x',
    '4136' => '--s-wxrw-',
    '4137' => '--s-wxrwx',
    '4140' => '--sr-----',
    '4141' => '--sr----x',
    '4142' => '--sr---w-',
    '4143' => '--sr---wx',
    '4144' => '--sr--r--',
    '4145' => '--sr--r-x',
    '4146' => '--sr--rw-',
    '4147' => '--sr--rwx',
    '4150' => '--sr-x---',
    '4151' => '--sr-x--x',
    '4152' => '--sr-x-w-',
    '4153' => '--sr-x-wx',
    '4154' => '--sr-xr--',
    '4155' => '--sr-xr-x',
    '4156' => '--sr-xrw-',
    '4157' => '--sr-xrwx',
    '4160' => '--srw----',
    '4161' => '--srw---x',
    '4162' => '--srw--w-',
    '4163' => '--srw--wx',
    '4164' => '--srw-r--',
    '4165' => '--srw-r-x',
    '4166' => '--srw-rw-',
    '4167' => '--srw-rwx',
    '4170' => '--srwx---',
    '4171' => '--srwx--x',
    '4172' => '--srwx-w-',
    '4173' => '--srwx-wx',
    '4174' => '--srwxr--',
    '4175' => '--srwxr-x',
    '4176' => '--srwxrw-',
    '4177' => '--srwxrwx',
    '4200' => '-wS------',
    '4201' => '-wS-----x',
    '4202' => '-wS----w-',
    '4203' => '-wS----wx',
    '4204' => '-wS---r--',
    '4205' => '-wS---r-x',
    '4206' => '-wS---rw-',
    '4207' => '-wS---rwx',
    '4210' => '-wS--x---',
    '4211' => '-wS--x--x',
    '4212' => '-wS--x-w-',
    '4213' => '-wS--x-wx',
    '4214' => '-wS--xr--',
    '4215' => '-wS--xr-x',
    '4216' => '-wS--xrw-',
    '4217' => '-wS--xrwx',
    '4220' => '-wS-w----',
    '4221' => '-wS-w---x',
    '4222' => '-wS-w--w-',
    '4223' => '-wS-w--wx',
    '4224' => '-wS-w-r--',
    '4225' => '-wS-w-r-x',
    '4226' => '-wS-w-rw-',
    '4227' => '-wS-w-rwx',
    '4230' => '-wS-wx---',
    '4231' => '-wS-wx--x',
    '4232' => '-wS-wx-w-',
    '4233' => '-wS-wx-wx',
    '4234' => '-wS-wxr--',
    '4235' => '-wS-wxr-x',
    '4236' => '-wS-wxrw-',
    '4237' => '-wS-wxrwx',
    '4240' => '-wSr-----',
    '4241' => '-wSr----x',
    '4242' => '-wSr---w-',
    '4243' => '-wSr---wx',
    '4244' => '-wSr--r--',
    '4245' => '-wSr--r-x',
    '4246' => '-wSr--rw-',
    '4247' => '-wSr--rwx',
    '4250' => '-wSr-x---',
    '4251' => '-wSr-x--x',
    '4252' => '-wSr-x-w-',
    '4253' => '-wSr-x-wx',
    '4254' => '-wSr-xr--',
    '4255' => '-wSr-xr-x',
    '4256' => '-wSr-xrw-',
    '4257' => '-wSr-xrwx',
    '4260' => '-wSrw----',
    '4261' => '-wSrw---x',
    '4262' => '-wSrw--w-',
    '4263' => '-wSrw--wx',
    '4264' => '-wSrw-r--',
    '4265' => '-wSrw-r-x',
    '4266' => '-wSrw-rw-',
    '4267' => '-wSrw-rwx',
    '4270' => '-wSrwx---',
    '4271' => '-wSrwx--x',
    '4272' => '-wSrwx-w-',
    '4273' => '-wSrwx-wx',
    '4274' => '-wSrwxr--',
    '4275' => '-wSrwxr-x',
    '4276' => '-wSrwxrw-',
    '4277' => '-wSrwxrwx',
    '4300' => '-ws------',
    '4301' => '-ws-----x',
    '4302' => '-ws----w-',
    '4303' => '-ws----wx',
    '4304' => '-ws---r--',
    '4305' => '-ws---r-x',
    '4306' => '-ws---rw-',
    '4307' => '-ws---rwx',
    '4310' => '-ws--x---',
    '4311' => '-ws--x--x',
    '4312' => '-ws--x-w-',
    '4313' => '-ws--x-wx',
    '4314' => '-ws--xr--',
    '4315' => '-ws--xr-x',
    '4316' => '-ws--xrw-',
    '4317' => '-ws--xrwx',
    '4320' => '-ws-w----',
    '4321' => '-ws-w---x',
    '4322' => '-ws-w--w-',
    '4323' => '-ws-w--wx',
    '4324' => '-ws-w-r--',
    '4325' => '-ws-w-r-x',
    '4326' => '-ws-w-rw-',
    '4327' => '-ws-w-rwx',
    '4330' => '-ws-wx---',
    '4331' => '-ws-wx--x',
    '4332' => '-ws-wx-w-',
    '4333' => '-ws-wx-wx',
    '4334' => '-ws-wxr--',
    '4335' => '-ws-wxr-x',
    '4336' => '-ws-wxrw-',
    '4337' => '-ws-wxrwx',
    '4340' => '-wsr-----',
    '4341' => '-wsr----x',
    '4342' => '-wsr---w-',
    '4343' => '-wsr---wx',
    '4344' => '-wsr--r--',
    '4345' => '-wsr--r-x',
    '4346' => '-wsr--rw-',
    '4347' => '-wsr--rwx',
    '4350' => '-wsr-x---',
    '4351' => '-wsr-x--x',
    '4352' => '-wsr-x-w-',
    '4353' => '-wsr-x-wx',
    '4354' => '-wsr-xr--',
    '4355' => '-wsr-xr-x',
    '4356' => '-wsr-xrw-',
    '4357' => '-wsr-xrwx',
    '4360' => '-wsrw----',
    '4361' => '-wsrw---x',
    '4362' => '-wsrw--w-',
    '4363' => '-wsrw--wx',
    '4364' => '-wsrw-r--',
    '4365' => '-wsrw-r-x',
    '4366' => '-wsrw-rw-',
    '4367' => '-wsrw-rwx',
    '4370' => '-wsrwx---',
    '4371' => '-wsrwx--x',
    '4372' => '-wsrwx-w-',
    '4373' => '-wsrwx-wx',
    '4374' => '-wsrwxr--',
    '4375' => '-wsrwxr-x',
    '4376' => '-wsrwxrw-',
    '4377' => '-wsrwxrwx',
    '4400' => 'r-S------',
    '4401' => 'r-S-----x',
    '4402' => 'r-S----w-',
    '4403' => 'r-S----wx',
    '4404' => 'r-S---r--',
    '4405' => 'r-S---r-x',
    '4406' => 'r-S---rw-',
    '4407' => 'r-S---rwx',
    '4410' => 'r-S--x---',
    '4411' => 'r-S--x--x',
    '4412' => 'r-S--x-w-',
    '4413' => 'r-S--x-wx',
    '4414' => 'r-S--xr--',
    '4415' => 'r-S--xr-x',
    '4416' => 'r-S--xrw-',
    '4417' => 'r-S--xrwx',
    '4420' => 'r-S-w----',
    '4421' => 'r-S-w---x',
    '4422' => 'r-S-w--w-',
    '4423' => 'r-S-w--wx',
    '4424' => 'r-S-w-r--',
    '4425' => 'r-S-w-r-x',
    '4426' => 'r-S-w-rw-',
    '4427' => 'r-S-w-rwx',
    '4430' => 'r-S-wx---',
    '4431' => 'r-S-wx--x',
    '4432' => 'r-S-wx-w-',
    '4433' => 'r-S-wx-wx',
    '4434' => 'r-S-wxr--',
    '4435' => 'r-S-wxr-x',
    '4436' => 'r-S-wxrw-',
    '4437' => 'r-S-wxrwx',
    '4440' => 'r-Sr-----',
    '4441' => 'r-Sr----x',
    '4442' => 'r-Sr---w-',
    '4443' => 'r-Sr---wx',
    '4444' => 'r-Sr--r--',
    '4445' => 'r-Sr--r-x',
    '4446' => 'r-Sr--rw-',
    '4447' => 'r-Sr--rwx',
    '4450' => 'r-Sr-x---',
    '4451' => 'r-Sr-x--x',
    '4452' => 'r-Sr-x-w-',
    '4453' => 'r-Sr-x-wx',
    '4454' => 'r-Sr-xr--',
    '4455' => 'r-Sr-xr-x',
    '4456' => 'r-Sr-xrw-',
    '4457' => 'r-Sr-xrwx',
    '4460' => 'r-Srw----',
    '4461' => 'r-Srw---x',
    '4462' => 'r-Srw--w-',
    '4463' => 'r-Srw--wx',
    '4464' => 'r-Srw-r--',
    '4465' => 'r-Srw-r-x',
    '4466' => 'r-Srw-rw-',
    '4467' => 'r-Srw-rwx',
    '4470' => 'r-Srwx---',
    '4471' => 'r-Srwx--x',
    '4472' => 'r-Srwx-w-',
    '4473' => 'r-Srwx-wx',
    '4474' => 'r-Srwxr--',
    '4475' => 'r-Srwxr-x',
    '4476' => 'r-Srwxrw-',
    '4477' => 'r-Srwxrwx',
    '4500' => 'r-s------',
    '4501' => 'r-s-----x',
    '4502' => 'r-s----w-',
    '4503' => 'r-s----wx',
    '4504' => 'r-s---r--',
    '4505' => 'r-s---r-x',
    '4506' => 'r-s---rw-',
    '4507' => 'r-s---rwx',
    '4510' => 'r-s--x---',
    '4511' => 'r-s--x--x',
    '4512' => 'r-s--x-w-',
    '4513' => 'r-s--x-wx',
    '4514' => 'r-s--xr--',
    '4515' => 'r-s--xr-x',
    '4516' => 'r-s--xrw-',
    '4517' => 'r-s--xrwx',
    '4520' => 'r-s-w----',
    '4521' => 'r-s-w---x',
    '4522' => 'r-s-w--w-',
    '4523' => 'r-s-w--wx',
    '4524' => 'r-s-w-r--',
    '4525' => 'r-s-w-r-x',
    '4526' => 'r-s-w-rw-',
    '4527' => 'r-s-w-rwx',
    '4530' => 'r-s-wx---',
    '4531' => 'r-s-wx--x',
    '4532' => 'r-s-wx-w-',
    '4533' => 'r-s-wx-wx',
    '4534' => 'r-s-wxr--',
    '4535' => 'r-s-wxr-x',
    '4536' => 'r-s-wxrw-',
    '4537' => 'r-s-wxrwx',
    '4540' => 'r-sr-----',
    '4541' => 'r-sr----x',
    '4542' => 'r-sr---w-',
    '4543' => 'r-sr---wx',
    '4544' => 'r-sr--r--',
    '4545' => 'r-sr--r-x',
    '4546' => 'r-sr--rw-',
    '4547' => 'r-sr--rwx',
    '4550' => 'r-sr-x---',
    '4551' => 'r-sr-x--x',
    '4552' => 'r-sr-x-w-',
    '4553' => 'r-sr-x-wx',
    '4554' => 'r-sr-xr--',
    '4555' => 'r-sr-xr-x',
    '4556' => 'r-sr-xrw-',
    '4557' => 'r-sr-xrwx',
    '4560' => 'r-srw----',
    '4561' => 'r-srw---x',
    '4562' => 'r-srw--w-',
    '4563' => 'r-srw--wx',
    '4564' => 'r-srw-r--',
    '4565' => 'r-srw-r-x',
    '4566' => 'r-srw-rw-',
    '4567' => 'r-srw-rwx',
    '4570' => 'r-srwx---',
    '4571' => 'r-srwx--x',
    '4572' => 'r-srwx-w-',
    '4573' => 'r-srwx-wx',
    '4574' => 'r-srwxr--',
    '4575' => 'r-srwxr-x',
    '4576' => 'r-srwxrw-',
    '4577' => 'r-srwxrwx',
    '4600' => 'rwS------',
    '4601' => 'rwS-----x',
    '4602' => 'rwS----w-',
    '4603' => 'rwS----wx',
    '4604' => 'rwS---r--',
    '4605' => 'rwS---r-x',
    '4606' => 'rwS---rw-',
    '4607' => 'rwS---rwx',
    '4610' => 'rwS--x---',
    '4611' => 'rwS--x--x',
    '4612' => 'rwS--x-w-',
    '4613' => 'rwS--x-wx',
    '4614' => 'rwS--xr--',
    '4615' => 'rwS--xr-x',
    '4616' => 'rwS--xrw-',
    '4617' => 'rwS--xrwx',
    '4620' => 'rwS-w----',
    '4621' => 'rwS-w---x',
    '4622' => 'rwS-w--w-',
    '4623' => 'rwS-w--wx',
    '4624' => 'rwS-w-r--',
    '4625' => 'rwS-w-r-x',
    '4626' => 'rwS-w-rw-',
    '4627' => 'rwS-w-rwx',
    '4630' => 'rwS-wx---',
    '4631' => 'rwS-wx--x',
    '4632' => 'rwS-wx-w-',
    '4633' => 'rwS-wx-wx',
    '4634' => 'rwS-wxr--',
    '4635' => 'rwS-wxr-x',
    '4636' => 'rwS-wxrw-',
    '4637' => 'rwS-wxrwx',
    '4640' => 'rwSr-----',
    '4641' => 'rwSr----x',
    '4642' => 'rwSr---w-',
    '4643' => 'rwSr---wx',
    '4644' => 'rwSr--r--',
    '4645' => 'rwSr--r-x',
    '4646' => 'rwSr--rw-',
    '4647' => 'rwSr--rwx',
    '4650' => 'rwSr-x---',
    '4651' => 'rwSr-x--x',
    '4652' => 'rwSr-x-w-',
    '4653' => 'rwSr-x-wx',
    '4654' => 'rwSr-xr--',
    '4655' => 'rwSr-xr-x',
    '4656' => 'rwSr-xrw-',
    '4657' => 'rwSr-xrwx',
    '4660' => 'rwSrw----',
    '4661' => 'rwSrw---x',
    '4662' => 'rwSrw--w-',
    '4663' => 'rwSrw--wx',
    '4664' => 'rwSrw-r--',
    '4665' => 'rwSrw-r-x',
    '4666' => 'rwSrw-rw-',
    '4667' => 'rwSrw-rwx',
    '4670' => 'rwSrwx---',
    '4671' => 'rwSrwx--x',
    '4672' => 'rwSrwx-w-',
    '4673' => 'rwSrwx-wx',
    '4674' => 'rwSrwxr--',
    '4675' => 'rwSrwxr-x',
    '4676' => 'rwSrwxrw-',
    '4677' => 'rwSrwxrwx',
    '4700' => 'rws------',
    '4701' => 'rws-----x',
    '4702' => 'rws----w-',
    '4703' => 'rws----wx',
    '4704' => 'rws---r--',
    '4705' => 'rws---r-x',
    '4706' => 'rws---rw-',
    '4707' => 'rws---rwx',
    '4710' => 'rws--x---',
    '4711' => 'rws--x--x',
    '4712' => 'rws--x-w-',
    '4713' => 'rws--x-wx',
    '4714' => 'rws--xr--',
    '4715' => 'rws--xr-x',
    '4716' => 'rws--xrw-',
    '4717' => 'rws--xrwx',
    '4720' => 'rws-w----',
    '4721' => 'rws-w---x',
    '4722' => 'rws-w--w-',
    '4723' => 'rws-w--wx',
    '4724' => 'rws-w-r--',
    '4725' => 'rws-w-r-x',
    '4726' => 'rws-w-rw-',
    '4727' => 'rws-w-rwx',
    '4730' => 'rws-wx---',
    '4731' => 'rws-wx--x',
    '4732' => 'rws-wx-w-',
    '4733' => 'rws-wx-wx',
    '4734' => 'rws-wxr--',
    '4735' => 'rws-wxr-x',
    '4736' => 'rws-wxrw-',
    '4737' => 'rws-wxrwx',
    '4740' => 'rwsr-----',
    '4741' => 'rwsr----x',
    '4742' => 'rwsr---w-',
    '4743' => 'rwsr---wx',
    '4744' => 'rwsr--r--',
    '4745' => 'rwsr--r-x',
    '4746' => 'rwsr--rw-',
    '4747' => 'rwsr--rwx',
    '4750' => 'rwsr-x---',
    '4751' => 'rwsr-x--x',
    '4752' => 'rwsr-x-w-',
    '4753' => 'rwsr-x-wx',
    '4754' => 'rwsr-xr--',
    '4755' => 'rwsr-xr-x',
    '4756' => 'rwsr-xrw-',
    '4757' => 'rwsr-xrwx',
    '4760' => 'rwsrw----',
    '4761' => 'rwsrw---x',
    '4762' => 'rwsrw--w-',
    '4763' => 'rwsrw--wx',
    '4764' => 'rwsrw-r--',
    '4765' => 'rwsrw-r-x',
    '4766' => 'rwsrw-rw-',
    '4767' => 'rwsrw-rwx',
    '4770' => 'rwsrwx---',
    '4771' => 'rwsrwx--x',
    '4772' => 'rwsrwx-w-',
    '4773' => 'rwsrwx-wx',
    '4774' => 'rwsrwxr--',
    '4775' => 'rwsrwxr-x',
    '4776' => 'rwsrwxrw-',
    '4777' => 'rwsrwxrwx',
    '5000' => '--S-----T',
    '5001' => '--S-----t',
    '5002' => '--S----wT',
    '5003' => '--S----wt',
    '5004' => '--S---r-T',
    '5005' => '--S---r-t',
    '5006' => '--S---rwT',
    '5007' => '--S---rwt',
    '5010' => '--S--x--T',
    '5011' => '--S--x--t',
    '5012' => '--S--x-wT',
    '5013' => '--S--x-wt',
    '5014' => '--S--xr-T',
    '5015' => '--S--xr-t',
    '5016' => '--S--xrwT',
    '5017' => '--S--xrwt',
    '5020' => '--S-w---T',
    '5021' => '--S-w---t',
    '5022' => '--S-w--wT',
    '5023' => '--S-w--wt',
    '5024' => '--S-w-r-T',
    '5025' => '--S-w-r-t',
    '5026' => '--S-w-rwT',
    '5027' => '--S-w-rwt',
    '5030' => '--S-wx--T',
    '5031' => '--S-wx--t',
    '5032' => '--S-wx-wT',
    '5033' => '--S-wx-wt',
    '5034' => '--S-wxr-T',
    '5035' => '--S-wxr-t',
    '5036' => '--S-wxrwT',
    '5037' => '--S-wxrwt',
    '5040' => '--Sr----T',
    '5041' => '--Sr----t',
    '5042' => '--Sr---wT',
    '5043' => '--Sr---wt',
    '5044' => '--Sr--r-T',
    '5045' => '--Sr--r-t',
    '5046' => '--Sr--rwT',
    '5047' => '--Sr--rwt',
    '5050' => '--Sr-x--T',
    '5051' => '--Sr-x--t',
    '5052' => '--Sr-x-wT',
    '5053' => '--Sr-x-wt',
    '5054' => '--Sr-xr-T',
    '5055' => '--Sr-xr-t',
    '5056' => '--Sr-xrwT',
    '5057' => '--Sr-xrwt',
    '5060' => '--Srw---T',
    '5061' => '--Srw---t',
    '5062' => '--Srw--wT',
    '5063' => '--Srw--wt',
    '5064' => '--Srw-r-T',
    '5065' => '--Srw-r-t',
    '5066' => '--Srw-rwT',
    '5067' => '--Srw-rwt',
    '5070' => '--Srwx--T',
    '5071' => '--Srwx--t',
    '5072' => '--Srwx-wT',
    '5073' => '--Srwx-wt',
    '5074' => '--Srwxr-T',
    '5075' => '--Srwxr-t',
    '5076' => '--SrwxrwT',
    '5077' => '--Srwxrwt',
    '5100' => '--s-----T',
    '5101' => '--s-----t',
    '5102' => '--s----wT',
    '5103' => '--s----wt',
    '5104' => '--s---r-T',
    '5105' => '--s---r-t',
    '5106' => '--s---rwT',
    '5107' => '--s---rwt',
    '5110' => '--s--x--T',
    '5111' => '--s--x--t',
    '5112' => '--s--x-wT',
    '5113' => '--s--x-wt',
    '5114' => '--s--xr-T',
    '5115' => '--s--xr-t',
    '5116' => '--s--xrwT',
    '5117' => '--s--xrwt',
    '5120' => '--s-w---T',
    '5121' => '--s-w---t',
    '5122' => '--s-w--wT',
    '5123' => '--s-w--wt',
    '5124' => '--s-w-r-T',
    '5125' => '--s-w-r-t',
    '5126' => '--s-w-rwT',
    '5127' => '--s-w-rwt',
    '5130' => '--s-wx--T',
    '5131' => '--s-wx--t',
    '5132' => '--s-wx-wT',
    '5133' => '--s-wx-wt',
    '5134' => '--s-wxr-T',
    '5135' => '--s-wxr-t',
    '5136' => '--s-wxrwT',
    '5137' => '--s-wxrwt',
    '5140' => '--sr----T',
    '5141' => '--sr----t',
    '5142' => '--sr---wT',
    '5143' => '--sr---wt',
    '5144' => '--sr--r-T',
    '5145' => '--sr--r-t',
    '5146' => '--sr--rwT',
    '5147' => '--sr--rwt',
    '5150' => '--sr-x--T',
    '5151' => '--sr-x--t',
    '5152' => '--sr-x-wT',
    '5153' => '--sr-x-wt',
    '5154' => '--sr-xr-T',
    '5155' => '--sr-xr-t',
    '5156' => '--sr-xrwT',
    '5157' => '--sr-xrwt',
    '5160' => '--srw---T',
    '5161' => '--srw---t',
    '5162' => '--srw--wT',
    '5163' => '--srw--wt',
    '5164' => '--srw-r-T',
    '5165' => '--srw-r-t',
    '5166' => '--srw-rwT',
    '5167' => '--srw-rwt',
    '5170' => '--srwx--T',
    '5171' => '--srwx--t',
    '5172' => '--srwx-wT',
    '5173' => '--srwx-wt',
    '5174' => '--srwxr-T',
    '5175' => '--srwxr-t',
    '5176' => '--srwxrwT',
    '5177' => '--srwxrwt',
    '5200' => '-wS-----T',
    '5201' => '-wS-----t',
    '5202' => '-wS----wT',
    '5203' => '-wS----wt',
    '5204' => '-wS---r-T',
    '5205' => '-wS---r-t',
    '5206' => '-wS---rwT',
    '5207' => '-wS---rwt',
    '5210' => '-wS--x--T',
    '5211' => '-wS--x--t',
    '5212' => '-wS--x-wT',
    '5213' => '-wS--x-wt',
    '5214' => '-wS--xr-T',
    '5215' => '-wS--xr-t',
    '5216' => '-wS--xrwT',
    '5217' => '-wS--xrwt',
    '5220' => '-wS-w---T',
    '5221' => '-wS-w---t',
    '5222' => '-wS-w--wT',
    '5223' => '-wS-w--wt',
    '5224' => '-wS-w-r-T',
    '5225' => '-wS-w-r-t',
    '5226' => '-wS-w-rwT',
    '5227' => '-wS-w-rwt',
    '5230' => '-wS-wx--T',
    '5231' => '-wS-wx--t',
    '5232' => '-wS-wx-wT',
    '5233' => '-wS-wx-wt',
    '5234' => '-wS-wxr-T',
    '5235' => '-wS-wxr-t',
    '5236' => '-wS-wxrwT',
    '5237' => '-wS-wxrwt',
    '5240' => '-wSr----T',
    '5241' => '-wSr----t',
    '5242' => '-wSr---wT',
    '5243' => '-wSr---wt',
    '5244' => '-wSr--r-T',
    '5245' => '-wSr--r-t',
    '5246' => '-wSr--rwT',
    '5247' => '-wSr--rwt',
    '5250' => '-wSr-x--T',
    '5251' => '-wSr-x--t',
    '5252' => '-wSr-x-wT',
    '5253' => '-wSr-x-wt',
    '5254' => '-wSr-xr-T',
    '5255' => '-wSr-xr-t',
    '5256' => '-wSr-xrwT',
    '5257' => '-wSr-xrwt',
    '5260' => '-wSrw---T',
    '5261' => '-wSrw---t',
    '5262' => '-wSrw--wT',
    '5263' => '-wSrw--wt',
    '5264' => '-wSrw-r-T',
    '5265' => '-wSrw-r-t',
    '5266' => '-wSrw-rwT',
    '5267' => '-wSrw-rwt',
    '5270' => '-wSrwx--T',
    '5271' => '-wSrwx--t',
    '5272' => '-wSrwx-wT',
    '5273' => '-wSrwx-wt',
    '5274' => '-wSrwxr-T',
    '5275' => '-wSrwxr-t',
    '5276' => '-wSrwxrwT',
    '5277' => '-wSrwxrwt',
    '5300' => '-ws-----T',
    '5301' => '-ws-----t',
    '5302' => '-ws----wT',
    '5303' => '-ws----wt',
    '5304' => '-ws---r-T',
    '5305' => '-ws---r-t',
    '5306' => '-ws---rwT',
    '5307' => '-ws---rwt',
    '5310' => '-ws--x--T',
    '5311' => '-ws--x--t',
    '5312' => '-ws--x-wT',
    '5313' => '-ws--x-wt',
    '5314' => '-ws--xr-T',
    '5315' => '-ws--xr-t',
    '5316' => '-ws--xrwT',
    '5317' => '-ws--xrwt',
    '5320' => '-ws-w---T',
    '5321' => '-ws-w---t',
    '5322' => '-ws-w--wT',
    '5323' => '-ws-w--wt',
    '5324' => '-ws-w-r-T',
    '5325' => '-ws-w-r-t',
    '5326' => '-ws-w-rwT',
    '5327' => '-ws-w-rwt',
    '5330' => '-ws-wx--T',
    '5331' => '-ws-wx--t',
    '5332' => '-ws-wx-wT',
    '5333' => '-ws-wx-wt',
    '5334' => '-ws-wxr-T',
    '5335' => '-ws-wxr-t',
    '5336' => '-ws-wxrwT',
    '5337' => '-ws-wxrwt',
    '5340' => '-wsr----T',
    '5341' => '-wsr----t',
    '5342' => '-wsr---wT',
    '5343' => '-wsr---wt',
    '5344' => '-wsr--r-T',
    '5345' => '-wsr--r-t',
    '5346' => '-wsr--rwT',
    '5347' => '-wsr--rwt',
    '5350' => '-wsr-x--T',
    '5351' => '-wsr-x--t',
    '5352' => '-wsr-x-wT',
    '5353' => '-wsr-x-wt',
    '5354' => '-wsr-xr-T',
    '5355' => '-wsr-xr-t',
    '5356' => '-wsr-xrwT',
    '5357' => '-wsr-xrwt',
    '5360' => '-wsrw---T',
    '5361' => '-wsrw---t',
    '5362' => '-wsrw--wT',
    '5363' => '-wsrw--wt',
    '5364' => '-wsrw-r-T',
    '5365' => '-wsrw-r-t',
    '5366' => '-wsrw-rwT',
    '5367' => '-wsrw-rwt',
    '5370' => '-wsrwx--T',
    '5371' => '-wsrwx--t',
    '5372' => '-wsrwx-wT',
    '5373' => '-wsrwx-wt',
    '5374' => '-wsrwxr-T',
    '5375' => '-wsrwxr-t',
    '5376' => '-wsrwxrwT',
    '5377' => '-wsrwxrwt',
    '5400' => 'r-S-----T',
    '5401' => 'r-S-----t',
    '5402' => 'r-S----wT',
    '5403' => 'r-S----wt',
    '5404' => 'r-S---r-T',
    '5405' => 'r-S---r-t',
    '5406' => 'r-S---rwT',
    '5407' => 'r-S---rwt',
    '5410' => 'r-S--x--T',
    '5411' => 'r-S--x--t',
    '5412' => 'r-S--x-wT',
    '5413' => 'r-S--x-wt',
    '5414' => 'r-S--xr-T',
    '5415' => 'r-S--xr-t',
    '5416' => 'r-S--xrwT',
    '5417' => 'r-S--xrwt',
    '5420' => 'r-S-w---T',
    '5421' => 'r-S-w---t',
    '5422' => 'r-S-w--wT',
    '5423' => 'r-S-w--wt',
    '5424' => 'r-S-w-r-T',
    '5425' => 'r-S-w-r-t',
    '5426' => 'r-S-w-rwT',
    '5427' => 'r-S-w-rwt',
    '5430' => 'r-S-wx--T',
    '5431' => 'r-S-wx--t',
    '5432' => 'r-S-wx-wT',
    '5433' => 'r-S-wx-wt',
    '5434' => 'r-S-wxr-T',
    '5435' => 'r-S-wxr-t',
    '5436' => 'r-S-wxrwT',
    '5437' => 'r-S-wxrwt',
    '5440' => 'r-Sr----T',
    '5441' => 'r-Sr----t',
    '5442' => 'r-Sr---wT',
    '5443' => 'r-Sr---wt',
    '5444' => 'r-Sr--r-T',
    '5445' => 'r-Sr--r-t',
    '5446' => 'r-Sr--rwT',
    '5447' => 'r-Sr--rwt',
    '5450' => 'r-Sr-x--T',
    '5451' => 'r-Sr-x--t',
    '5452' => 'r-Sr-x-wT',
    '5453' => 'r-Sr-x-wt',
    '5454' => 'r-Sr-xr-T',
    '5455' => 'r-Sr-xr-t',
    '5456' => 'r-Sr-xrwT',
    '5457' => 'r-Sr-xrwt',
    '5460' => 'r-Srw---T',
    '5461' => 'r-Srw---t',
    '5462' => 'r-Srw--wT',
    '5463' => 'r-Srw--wt',
    '5464' => 'r-Srw-r-T',
    '5465' => 'r-Srw-r-t',
    '5466' => 'r-Srw-rwT',
    '5467' => 'r-Srw-rwt',
    '5470' => 'r-Srwx--T',
    '5471' => 'r-Srwx--t',
    '5472' => 'r-Srwx-wT',
    '5473' => 'r-Srwx-wt',
    '5474' => 'r-Srwxr-T',
    '5475' => 'r-Srwxr-t',
    '5476' => 'r-SrwxrwT',
    '5477' => 'r-Srwxrwt',
    '5500' => 'r-s-----T',
    '5501' => 'r-s-----t',
    '5502' => 'r-s----wT',
    '5503' => 'r-s----wt',
    '5504' => 'r-s---r-T',
    '5505' => 'r-s---r-t',
    '5506' => 'r-s---rwT',
    '5507' => 'r-s---rwt',
    '5510' => 'r-s--x--T',
    '5511' => 'r-s--x--t',
    '5512' => 'r-s--x-wT',
    '5513' => 'r-s--x-wt',
    '5514' => 'r-s--xr-T',
    '5515' => 'r-s--xr-t',
    '5516' => 'r-s--xrwT',
    '5517' => 'r-s--xrwt',
    '5520' => 'r-s-w---T',
    '5521' => 'r-s-w---t',
    '5522' => 'r-s-w--wT',
    '5523' => 'r-s-w--wt',
    '5524' => 'r-s-w-r-T',
    '5525' => 'r-s-w-r-t',
    '5526' => 'r-s-w-rwT',
    '5527' => 'r-s-w-rwt',
    '5530' => 'r-s-wx--T',
    '5531' => 'r-s-wx--t',
    '5532' => 'r-s-wx-wT',
    '5533' => 'r-s-wx-wt',
    '5534' => 'r-s-wxr-T',
    '5535' => 'r-s-wxr-t',
    '5536' => 'r-s-wxrwT',
    '5537' => 'r-s-wxrwt',
    '5540' => 'r-sr----T',
    '5541' => 'r-sr----t',
    '5542' => 'r-sr---wT',
    '5543' => 'r-sr---wt',
    '5544' => 'r-sr--r-T',
    '5545' => 'r-sr--r-t',
    '5546' => 'r-sr--rwT',
    '5547' => 'r-sr--rwt',
    '5550' => 'r-sr-x--T',
    '5551' => 'r-sr-x--t',
    '5552' => 'r-sr-x-wT',
    '5553' => 'r-sr-x-wt',
    '5554' => 'r-sr-xr-T',
    '5555' => 'r-sr-xr-t',
    '5556' => 'r-sr-xrwT',
    '5557' => 'r-sr-xrwt',
    '5560' => 'r-srw---T',
    '5561' => 'r-srw---t',
    '5562' => 'r-srw--wT',
    '5563' => 'r-srw--wt',
    '5564' => 'r-srw-r-T',
    '5565' => 'r-srw-r-t',
    '5566' => 'r-srw-rwT',
    '5567' => 'r-srw-rwt',
    '5570' => 'r-srwx--T',
    '5571' => 'r-srwx--t',
    '5572' => 'r-srwx-wT',
    '5573' => 'r-srwx-wt',
    '5574' => 'r-srwxr-T',
    '5575' => 'r-srwxr-t',
    '5576' => 'r-srwxrwT',
    '5577' => 'r-srwxrwt',
    '5600' => 'rwS-----T',
    '5601' => 'rwS-----t',
    '5602' => 'rwS----wT',
    '5603' => 'rwS----wt',
    '5604' => 'rwS---r-T',
    '5605' => 'rwS---r-t',
    '5606' => 'rwS---rwT',
    '5607' => 'rwS---rwt',
    '5610' => 'rwS--x--T',
    '5611' => 'rwS--x--t',
    '5612' => 'rwS--x-wT',
    '5613' => 'rwS--x-wt',
    '5614' => 'rwS--xr-T',
    '5615' => 'rwS--xr-t',
    '5616' => 'rwS--xrwT',
    '5617' => 'rwS--xrwt',
    '5620' => 'rwS-w---T',
    '5621' => 'rwS-w---t',
    '5622' => 'rwS-w--wT',
    '5623' => 'rwS-w--wt',
    '5624' => 'rwS-w-r-T',
    '5625' => 'rwS-w-r-t',
    '5626' => 'rwS-w-rwT',
    '5627' => 'rwS-w-rwt',
    '5630' => 'rwS-wx--T',
    '5631' => 'rwS-wx--t',
    '5632' => 'rwS-wx-wT',
    '5633' => 'rwS-wx-wt',
    '5634' => 'rwS-wxr-T',
    '5635' => 'rwS-wxr-t',
    '5636' => 'rwS-wxrwT',
    '5637' => 'rwS-wxrwt',
    '5640' => 'rwSr----T',
    '5641' => 'rwSr----t',
    '5642' => 'rwSr---wT',
    '5643' => 'rwSr---wt',
    '5644' => 'rwSr--r-T',
    '5645' => 'rwSr--r-t',
    '5646' => 'rwSr--rwT',
    '5647' => 'rwSr--rwt',
    '5650' => 'rwSr-x--T',
    '5651' => 'rwSr-x--t',
    '5652' => 'rwSr-x-wT',
    '5653' => 'rwSr-x-wt',
    '5654' => 'rwSr-xr-T',
    '5655' => 'rwSr-xr-t',
    '5656' => 'rwSr-xrwT',
    '5657' => 'rwSr-xrwt',
    '5660' => 'rwSrw---T',
    '5661' => 'rwSrw---t',
    '5662' => 'rwSrw--wT',
    '5663' => 'rwSrw--wt',
    '5664' => 'rwSrw-r-T',
    '5665' => 'rwSrw-r-t',
    '5666' => 'rwSrw-rwT',
    '5667' => 'rwSrw-rwt',
    '5670' => 'rwSrwx--T',
    '5671' => 'rwSrwx--t',
    '5672' => 'rwSrwx-wT',
    '5673' => 'rwSrwx-wt',
    '5674' => 'rwSrwxr-T',
    '5675' => 'rwSrwxr-t',
    '5676' => 'rwSrwxrwT',
    '5677' => 'rwSrwxrwt',
    '5700' => 'rws-----T',
    '5701' => 'rws-----t',
    '5702' => 'rws----wT',
    '5703' => 'rws----wt',
    '5704' => 'rws---r-T',
    '5705' => 'rws---r-t',
    '5706' => 'rws---rwT',
    '5707' => 'rws---rwt',
    '5710' => 'rws--x--T',
    '5711' => 'rws--x--t',
    '5712' => 'rws--x-wT',
    '5713' => 'rws--x-wt',
    '5714' => 'rws--xr-T',
    '5715' => 'rws--xr-t',
    '5716' => 'rws--xrwT',
    '5717' => 'rws--xrwt',
    '5720' => 'rws-w---T',
    '5721' => 'rws-w---t',
    '5722' => 'rws-w--wT',
    '5723' => 'rws-w--wt',
    '5724' => 'rws-w-r-T',
    '5725' => 'rws-w-r-t',
    '5726' => 'rws-w-rwT',
    '5727' => 'rws-w-rwt',
    '5730' => 'rws-wx--T',
    '5731' => 'rws-wx--t',
    '5732' => 'rws-wx-wT',
    '5733' => 'rws-wx-wt',
    '5734' => 'rws-wxr-T',
    '5735' => 'rws-wxr-t',
    '5736' => 'rws-wxrwT',
    '5737' => 'rws-wxrwt',
    '5740' => 'rwsr----T',
    '5741' => 'rwsr----t',
    '5742' => 'rwsr---wT',
    '5743' => 'rwsr---wt',
    '5744' => 'rwsr--r-T',
    '5745' => 'rwsr--r-t',
    '5746' => 'rwsr--rwT',
    '5747' => 'rwsr--rwt',
    '5750' => 'rwsr-x--T',
    '5751' => 'rwsr-x--t',
    '5752' => 'rwsr-x-wT',
    '5753' => 'rwsr-x-wt',
    '5754' => 'rwsr-xr-T',
    '5755' => 'rwsr-xr-t',
    '5756' => 'rwsr-xrwT',
    '5757' => 'rwsr-xrwt',
    '5760' => 'rwsrw---T',
    '5761' => 'rwsrw---t',
    '5762' => 'rwsrw--wT',
    '5763' => 'rwsrw--wt',
    '5764' => 'rwsrw-r-T',
    '5765' => 'rwsrw-r-t',
    '5766' => 'rwsrw-rwT',
    '5767' => 'rwsrw-rwt',
    '5770' => 'rwsrwx--T',
    '5771' => 'rwsrwx--t',
    '5772' => 'rwsrwx-wT',
    '5773' => 'rwsrwx-wt',
    '5774' => 'rwsrwxr-T',
    '5775' => 'rwsrwxr-t',
    '5776' => 'rwsrwxrwT',
    '5777' => 'rwsrwxrwt',
    '6000' => '--S--S---',
    '6001' => '--S--S--x',
    '6002' => '--S--S-w-',
    '6003' => '--S--S-wx',
    '6004' => '--S--Sr--',
    '6005' => '--S--Sr-x',
    '6006' => '--S--Srw-',
    '6007' => '--S--Srwx',
    '6010' => '--S--s---',
    '6011' => '--S--s--x',
    '6012' => '--S--s-w-',
    '6013' => '--S--s-wx',
    '6014' => '--S--sr--',
    '6015' => '--S--sr-x',
    '6016' => '--S--srw-',
    '6017' => '--S--srwx',
    '6020' => '--S-wS---',
    '6021' => '--S-wS--x',
    '6022' => '--S-wS-w-',
    '6023' => '--S-wS-wx',
    '6024' => '--S-wSr--',
    '6025' => '--S-wSr-x',
    '6026' => '--S-wSrw-',
    '6027' => '--S-wSrwx',
    '6030' => '--S-ws---',
    '6031' => '--S-ws--x',
    '6032' => '--S-ws-w-',
    '6033' => '--S-ws-wx',
    '6034' => '--S-wsr--',
    '6035' => '--S-wsr-x',
    '6036' => '--S-wsrw-',
    '6037' => '--S-wsrwx',
    '6040' => '--Sr-S---',
    '6041' => '--Sr-S--x',
    '6042' => '--Sr-S-w-',
    '6043' => '--Sr-S-wx',
    '6044' => '--Sr-Sr--',
    '6045' => '--Sr-Sr-x',
    '6046' => '--Sr-Srw-',
    '6047' => '--Sr-Srwx',
    '6050' => '--Sr-s---',
    '6051' => '--Sr-s--x',
    '6052' => '--Sr-s-w-',
    '6053' => '--Sr-s-wx',
    '6054' => '--Sr-sr--',
    '6055' => '--Sr-sr-x',
    '6056' => '--Sr-srw-',
    '6057' => '--Sr-srwx',
    '6060' => '--SrwS---',
    '6061' => '--SrwS--x',
    '6062' => '--SrwS-w-',
    '6063' => '--SrwS-wx',
    '6064' => '--SrwSr--',
    '6065' => '--SrwSr-x',
    '6066' => '--SrwSrw-',
    '6067' => '--SrwSrwx',
    '6070' => '--Srws---',
    '6071' => '--Srws--x',
    '6072' => '--Srws-w-',
    '6073' => '--Srws-wx',
    '6074' => '--Srwsr--',
    '6075' => '--Srwsr-x',
    '6076' => '--Srwsrw-',
    '6077' => '--Srwsrwx',
    '6100' => '--s--S---',
    '6101' => '--s--S--x',
    '6102' => '--s--S-w-',
    '6103' => '--s--S-wx',
    '6104' => '--s--Sr--',
    '6105' => '--s--Sr-x',
    '6106' => '--s--Srw-',
    '6107' => '--s--Srwx',
    '6110' => '--s--s---',
    '6111' => '--s--s--x',
    '6112' => '--s--s-w-',
    '6113' => '--s--s-wx',
    '6114' => '--s--sr--',
    '6115' => '--s--sr-x',
    '6116' => '--s--srw-',
    '6117' => '--s--srwx',
    '6120' => '--s-wS---',
    '6121' => '--s-wS--x',
    '6122' => '--s-wS-w-',
    '6123' => '--s-wS-wx',
    '6124' => '--s-wSr--',
    '6125' => '--s-wSr-x',
    '6126' => '--s-wSrw-',
    '6127' => '--s-wSrwx',
    '6130' => '--s-ws---',
    '6131' => '--s-ws--x',
    '6132' => '--s-ws-w-',
    '6133' => '--s-ws-wx',
    '6134' => '--s-wsr--',
    '6135' => '--s-wsr-x',
    '6136' => '--s-wsrw-',
    '6137' => '--s-wsrwx',
    '6140' => '--sr-S---',
    '6141' => '--sr-S--x',
    '6142' => '--sr-S-w-',
    '6143' => '--sr-S-wx',
    '6144' => '--sr-Sr--',
    '6145' => '--sr-Sr-x',
    '6146' => '--sr-Srw-',
    '6147' => '--sr-Srwx',
    '6150' => '--sr-s---',
    '6151' => '--sr-s--x',
    '6152' => '--sr-s-w-',
    '6153' => '--sr-s-wx',
    '6154' => '--sr-sr--',
    '6155' => '--sr-sr-x',
    '6156' => '--sr-srw-',
    '6157' => '--sr-srwx',
    '6160' => '--srwS---',
    '6161' => '--srwS--x',
    '6162' => '--srwS-w-',
    '6163' => '--srwS-wx',
    '6164' => '--srwSr--',
    '6165' => '--srwSr-x',
    '6166' => '--srwSrw-',
    '6167' => '--srwSrwx',
    '6170' => '--srws---',
    '6171' => '--srws--x',
    '6172' => '--srws-w-',
    '6173' => '--srws-wx',
    '6174' => '--srwsr--',
    '6175' => '--srwsr-x',
    '6176' => '--srwsrw-',
    '6177' => '--srwsrwx',
    '6200' => '-wS--S---',
    '6201' => '-wS--S--x',
    '6202' => '-wS--S-w-',
    '6203' => '-wS--S-wx',
    '6204' => '-wS--Sr--',
    '6205' => '-wS--Sr-x',
    '6206' => '-wS--Srw-',
    '6207' => '-wS--Srwx',
    '6210' => '-wS--s---',
    '6211' => '-wS--s--x',
    '6212' => '-wS--s-w-',
    '6213' => '-wS--s-wx',
    '6214' => '-wS--sr--',
    '6215' => '-wS--sr-x',
    '6216' => '-wS--srw-',
    '6217' => '-wS--srwx',
    '6220' => '-wS-wS---',
    '6221' => '-wS-wS--x',
    '6222' => '-wS-wS-w-',
    '6223' => '-wS-wS-wx',
    '6224' => '-wS-wSr--',
    '6225' => '-wS-wSr-x',
    '6226' => '-wS-wSrw-',
    '6227' => '-wS-wSrwx',
    '6230' => '-wS-ws---',
    '6231' => '-wS-ws--x',
    '6232' => '-wS-ws-w-',
    '6233' => '-wS-ws-wx',
    '6234' => '-wS-wsr--',
    '6235' => '-wS-wsr-x',
    '6236' => '-wS-wsrw-',
    '6237' => '-wS-wsrwx',
    '6240' => '-wSr-S---',
    '6241' => '-wSr-S--x',
    '6242' => '-wSr-S-w-',
    '6243' => '-wSr-S-wx',
    '6244' => '-wSr-Sr--',
    '6245' => '-wSr-Sr-x',
    '6246' => '-wSr-Srw-',
    '6247' => '-wSr-Srwx',
    '6250' => '-wSr-s---',
    '6251' => '-wSr-s--x',
    '6252' => '-wSr-s-w-',
    '6253' => '-wSr-s-wx',
    '6254' => '-wSr-sr--',
    '6255' => '-wSr-sr-x',
    '6256' => '-wSr-srw-',
    '6257' => '-wSr-srwx',
    '6260' => '-wSrwS---',
    '6261' => '-wSrwS--x',
    '6262' => '-wSrwS-w-',
    '6263' => '-wSrwS-wx',
    '6264' => '-wSrwSr--',
    '6265' => '-wSrwSr-x',
    '6266' => '-wSrwSrw-',
    '6267' => '-wSrwSrwx',
    '6270' => '-wSrws---',
    '6271' => '-wSrws--x',
    '6272' => '-wSrws-w-',
    '6273' => '-wSrws-wx',
    '6274' => '-wSrwsr--',
    '6275' => '-wSrwsr-x',
    '6276' => '-wSrwsrw-',
    '6277' => '-wSrwsrwx',
    '6300' => '-ws--S---',
    '6301' => '-ws--S--x',
    '6302' => '-ws--S-w-',
    '6303' => '-ws--S-wx',
    '6304' => '-ws--Sr--',
    '6305' => '-ws--Sr-x',
    '6306' => '-ws--Srw-',
    '6307' => '-ws--Srwx',
    '6310' => '-ws--s---',
    '6311' => '-ws--s--x',
    '6312' => '-ws--s-w-',
    '6313' => '-ws--s-wx',
    '6314' => '-ws--sr--',
    '6315' => '-ws--sr-x',
    '6316' => '-ws--srw-',
    '6317' => '-ws--srwx',
    '6320' => '-ws-wS---',
    '6321' => '-ws-wS--x',
    '6322' => '-ws-wS-w-',
    '6323' => '-ws-wS-wx',
    '6324' => '-ws-wSr--',
    '6325' => '-ws-wSr-x',
    '6326' => '-ws-wSrw-',
    '6327' => '-ws-wSrwx',
    '6330' => '-ws-ws---',
    '6331' => '-ws-ws--x',
    '6332' => '-ws-ws-w-',
    '6333' => '-ws-ws-wx',
    '6334' => '-ws-wsr--',
    '6335' => '-ws-wsr-x',
    '6336' => '-ws-wsrw-',
    '6337' => '-ws-wsrwx',
    '6340' => '-wsr-S---',
    '6341' => '-wsr-S--x',
    '6342' => '-wsr-S-w-',
    '6343' => '-wsr-S-wx',
    '6344' => '-wsr-Sr--',
    '6345' => '-wsr-Sr-x',
    '6346' => '-wsr-Srw-',
    '6347' => '-wsr-Srwx',
    '6350' => '-wsr-s---',
    '6351' => '-wsr-s--x',
    '6352' => '-wsr-s-w-',
    '6353' => '-wsr-s-wx',
    '6354' => '-wsr-sr--',
    '6355' => '-wsr-sr-x',
    '6356' => '-wsr-srw-',
    '6357' => '-wsr-srwx',
    '6360' => '-wsrwS---',
    '6361' => '-wsrwS--x',
    '6362' => '-wsrwS-w-',
    '6363' => '-wsrwS-wx',
    '6364' => '-wsrwSr--',
    '6365' => '-wsrwSr-x',
    '6366' => '-wsrwSrw-',
    '6367' => '-wsrwSrwx',
    '6370' => '-wsrws---',
    '6371' => '-wsrws--x',
    '6372' => '-wsrws-w-',
    '6373' => '-wsrws-wx',
    '6374' => '-wsrwsr--',
    '6375' => '-wsrwsr-x',
    '6376' => '-wsrwsrw-',
    '6377' => '-wsrwsrwx',
    '6400' => 'r-S--S---',
    '6401' => 'r-S--S--x',
    '6402' => 'r-S--S-w-',
    '6403' => 'r-S--S-wx',
    '6404' => 'r-S--Sr--',
    '6405' => 'r-S--Sr-x',
    '6406' => 'r-S--Srw-',
    '6407' => 'r-S--Srwx',
    '6410' => 'r-S--s---',
    '6411' => 'r-S--s--x',
    '6412' => 'r-S--s-w-',
    '6413' => 'r-S--s-wx',
    '6414' => 'r-S--sr--',
    '6415' => 'r-S--sr-x',
    '6416' => 'r-S--srw-',
    '6417' => 'r-S--srwx',
    '6420' => 'r-S-wS---',
    '6421' => 'r-S-wS--x',
    '6422' => 'r-S-wS-w-',
    '6423' => 'r-S-wS-wx',
    '6424' => 'r-S-wSr--',
    '6425' => 'r-S-wSr-x',
    '6426' => 'r-S-wSrw-',
    '6427' => 'r-S-wSrwx',
    '6430' => 'r-S-ws---',
    '6431' => 'r-S-ws--x',
    '6432' => 'r-S-ws-w-',
    '6433' => 'r-S-ws-wx',
    '6434' => 'r-S-wsr--',
    '6435' => 'r-S-wsr-x',
    '6436' => 'r-S-wsrw-',
    '6437' => 'r-S-wsrwx',
    '6440' => 'r-Sr-S---',
    '6441' => 'r-Sr-S--x',
    '6442' => 'r-Sr-S-w-',
    '6443' => 'r-Sr-S-wx',
    '6444' => 'r-Sr-Sr--',
    '6445' => 'r-Sr-Sr-x',
    '6446' => 'r-Sr-Srw-',
    '6447' => 'r-Sr-Srwx',
    '6450' => 'r-Sr-s---',
    '6451' => 'r-Sr-s--x',
    '6452' => 'r-Sr-s-w-',
    '6453' => 'r-Sr-s-wx',
    '6454' => 'r-Sr-sr--',
    '6455' => 'r-Sr-sr-x',
    '6456' => 'r-Sr-srw-',
    '6457' => 'r-Sr-srwx',
    '6460' => 'r-SrwS---',
    '6461' => 'r-SrwS--x',
    '6462' => 'r-SrwS-w-',
    '6463' => 'r-SrwS-wx',
    '6464' => 'r-SrwSr--',
    '6465' => 'r-SrwSr-x',
    '6466' => 'r-SrwSrw-',
    '6467' => 'r-SrwSrwx',
    '6470' => 'r-Srws---',
    '6471' => 'r-Srws--x',
    '6472' => 'r-Srws-w-',
    '6473' => 'r-Srws-wx',
    '6474' => 'r-Srwsr--',
    '6475' => 'r-Srwsr-x',
    '6476' => 'r-Srwsrw-',
    '6477' => 'r-Srwsrwx',
    '6500' => 'r-s--S---',
    '6501' => 'r-s--S--x',
    '6502' => 'r-s--S-w-',
    '6503' => 'r-s--S-wx',
    '6504' => 'r-s--Sr--',
    '6505' => 'r-s--Sr-x',
    '6506' => 'r-s--Srw-',
    '6507' => 'r-s--Srwx',
    '6510' => 'r-s--s---',
    '6511' => 'r-s--s--x',
    '6512' => 'r-s--s-w-',
    '6513' => 'r-s--s-wx',
    '6514' => 'r-s--sr--',
    '6515' => 'r-s--sr-x',
    '6516' => 'r-s--srw-',
    '6517' => 'r-s--srwx',
    '6520' => 'r-s-wS---',
    '6521' => 'r-s-wS--x',
    '6522' => 'r-s-wS-w-',
    '6523' => 'r-s-wS-wx',
    '6524' => 'r-s-wSr--',
    '6525' => 'r-s-wSr-x',
    '6526' => 'r-s-wSrw-',
    '6527' => 'r-s-wSrwx',
    '6530' => 'r-s-ws---',
    '6531' => 'r-s-ws--x',
    '6532' => 'r-s-ws-w-',
    '6533' => 'r-s-ws-wx',
    '6534' => 'r-s-wsr--',
    '6535' => 'r-s-wsr-x',
    '6536' => 'r-s-wsrw-',
    '6537' => 'r-s-wsrwx',
    '6540' => 'r-sr-S---',
    '6541' => 'r-sr-S--x',
    '6542' => 'r-sr-S-w-',
    '6543' => 'r-sr-S-wx',
    '6544' => 'r-sr-Sr--',
    '6545' => 'r-sr-Sr-x',
    '6546' => 'r-sr-Srw-',
    '6547' => 'r-sr-Srwx',
    '6550' => 'r-sr-s---',
    '6551' => 'r-sr-s--x',
    '6552' => 'r-sr-s-w-',
    '6553' => 'r-sr-s-wx',
    '6554' => 'r-sr-sr--',
    '6555' => 'r-sr-sr-x',
    '6556' => 'r-sr-srw-',
    '6557' => 'r-sr-srwx',
    '6560' => 'r-srwS---',
    '6561' => 'r-srwS--x',
    '6562' => 'r-srwS-w-',
    '6563' => 'r-srwS-wx',
    '6564' => 'r-srwSr--',
    '6565' => 'r-srwSr-x',
    '6566' => 'r-srwSrw-',
    '6567' => 'r-srwSrwx',
    '6570' => 'r-srws---',
    '6571' => 'r-srws--x',
    '6572' => 'r-srws-w-',
    '6573' => 'r-srws-wx',
    '6574' => 'r-srwsr--',
    '6575' => 'r-srwsr-x',
    '6576' => 'r-srwsrw-',
    '6577' => 'r-srwsrwx',
    '6600' => 'rwS--S---',
    '6601' => 'rwS--S--x',
    '6602' => 'rwS--S-w-',
    '6603' => 'rwS--S-wx',
    '6604' => 'rwS--Sr--',
    '6605' => 'rwS--Sr-x',
    '6606' => 'rwS--Srw-',
    '6607' => 'rwS--Srwx',
    '6610' => 'rwS--s---',
    '6611' => 'rwS--s--x',
    '6612' => 'rwS--s-w-',
    '6613' => 'rwS--s-wx',
    '6614' => 'rwS--sr--',
    '6615' => 'rwS--sr-x',
    '6616' => 'rwS--srw-',
    '6617' => 'rwS--srwx',
    '6620' => 'rwS-wS---',
    '6621' => 'rwS-wS--x',
    '6622' => 'rwS-wS-w-',
    '6623' => 'rwS-wS-wx',
    '6624' => 'rwS-wSr--',
    '6625' => 'rwS-wSr-x',
    '6626' => 'rwS-wSrw-',
    '6627' => 'rwS-wSrwx',
    '6630' => 'rwS-ws---',
    '6631' => 'rwS-ws--x',
    '6632' => 'rwS-ws-w-',
    '6633' => 'rwS-ws-wx',
    '6634' => 'rwS-wsr--',
    '6635' => 'rwS-wsr-x',
    '6636' => 'rwS-wsrw-',
    '6637' => 'rwS-wsrwx',
    '6640' => 'rwSr-S---',
    '6641' => 'rwSr-S--x',
    '6642' => 'rwSr-S-w-',
    '6643' => 'rwSr-S-wx',
    '6644' => 'rwSr-Sr--',
    '6645' => 'rwSr-Sr-x',
    '6646' => 'rwSr-Srw-',
    '6647' => 'rwSr-Srwx',
    '6650' => 'rwSr-s---',
    '6651' => 'rwSr-s--x',
    '6652' => 'rwSr-s-w-',
    '6653' => 'rwSr-s-wx',
    '6654' => 'rwSr-sr--',
    '6655' => 'rwSr-sr-x',
    '6656' => 'rwSr-srw-',
    '6657' => 'rwSr-srwx',
    '6660' => 'rwSrwS---',
    '6661' => 'rwSrwS--x',
    '6662' => 'rwSrwS-w-',
    '6663' => 'rwSrwS-wx',
    '6664' => 'rwSrwSr--',
    '6665' => 'rwSrwSr-x',
    '6666' => 'rwSrwSrw-',
    '6667' => 'rwSrwSrwx',
    '6670' => 'rwSrws---',
    '6671' => 'rwSrws--x',
    '6672' => 'rwSrws-w-',
    '6673' => 'rwSrws-wx',
    '6674' => 'rwSrwsr--',
    '6675' => 'rwSrwsr-x',
    '6676' => 'rwSrwsrw-',
    '6677' => 'rwSrwsrwx',
    '6700' => 'rws--S---',
    '6701' => 'rws--S--x',
    '6702' => 'rws--S-w-',
    '6703' => 'rws--S-wx',
    '6704' => 'rws--Sr--',
    '6705' => 'rws--Sr-x',
    '6706' => 'rws--Srw-',
    '6707' => 'rws--Srwx',
    '6710' => 'rws--s---',
    '6711' => 'rws--s--x',
    '6712' => 'rws--s-w-',
    '6713' => 'rws--s-wx',
    '6714' => 'rws--sr--',
    '6715' => 'rws--sr-x',
    '6716' => 'rws--srw-',
    '6717' => 'rws--srwx',
    '6720' => 'rws-wS---',
    '6721' => 'rws-wS--x',
    '6722' => 'rws-wS-w-',
    '6723' => 'rws-wS-wx',
    '6724' => 'rws-wSr--',
    '6725' => 'rws-wSr-x',
    '6726' => 'rws-wSrw-',
    '6727' => 'rws-wSrwx',
    '6730' => 'rws-ws---',
    '6731' => 'rws-ws--x',
    '6732' => 'rws-ws-w-',
    '6733' => 'rws-ws-wx',
    '6734' => 'rws-wsr--',
    '6735' => 'rws-wsr-x',
    '6736' => 'rws-wsrw-',
    '6737' => 'rws-wsrwx',
    '6740' => 'rwsr-S---',
    '6741' => 'rwsr-S--x',
    '6742' => 'rwsr-S-w-',
    '6743' => 'rwsr-S-wx',
    '6744' => 'rwsr-Sr--',
    '6745' => 'rwsr-Sr-x',
    '6746' => 'rwsr-Srw-',
    '6747' => 'rwsr-Srwx',
    '6750' => 'rwsr-s---',
    '6751' => 'rwsr-s--x',
    '6752' => 'rwsr-s-w-',
    '6753' => 'rwsr-s-wx',
    '6754' => 'rwsr-sr--',
    '6755' => 'rwsr-sr-x',
    '6756' => 'rwsr-srw-',
    '6757' => 'rwsr-srwx',
    '6760' => 'rwsrwS---',
    '6761' => 'rwsrwS--x',
    '6762' => 'rwsrwS-w-',
    '6763' => 'rwsrwS-wx',
    '6764' => 'rwsrwSr--',
    '6765' => 'rwsrwSr-x',
    '6766' => 'rwsrwSrw-',
    '6767' => 'rwsrwSrwx',
    '6770' => 'rwsrws---',
    '6771' => 'rwsrws--x',
    '6772' => 'rwsrws-w-',
    '6773' => 'rwsrws-wx',
    '6774' => 'rwsrwsr--',
    '6775' => 'rwsrwsr-x',
    '6776' => 'rwsrwsrw-',
    '6777' => 'rwsrwsrwx',
    '7000' => '--S--S--T',
    '7001' => '--S--S--t',
    '7002' => '--S--S-wT',
    '7003' => '--S--S-wt',
    '7004' => '--S--Sr-T',
    '7005' => '--S--Sr-t',
    '7006' => '--S--SrwT',
    '7007' => '--S--Srwt',
    '7010' => '--S--s--T',
    '7011' => '--S--s--t',
    '7012' => '--S--s-wT',
    '7013' => '--S--s-wt',
    '7014' => '--S--sr-T',
    '7015' => '--S--sr-t',
    '7016' => '--S--srwT',
    '7017' => '--S--srwt',
    '7020' => '--S-wS--T',
    '7021' => '--S-wS--t',
    '7022' => '--S-wS-wT',
    '7023' => '--S-wS-wt',
    '7024' => '--S-wSr-T',
    '7025' => '--S-wSr-t',
    '7026' => '--S-wSrwT',
    '7027' => '--S-wSrwt',
    '7030' => '--S-ws--T',
    '7031' => '--S-ws--t',
    '7032' => '--S-ws-wT',
    '7033' => '--S-ws-wt',
    '7034' => '--S-wsr-T',
    '7035' => '--S-wsr-t',
    '7036' => '--S-wsrwT',
    '7037' => '--S-wsrwt',
    '7040' => '--Sr-S--T',
    '7041' => '--Sr-S--t',
    '7042' => '--Sr-S-wT',
    '7043' => '--Sr-S-wt',
    '7044' => '--Sr-Sr-T',
    '7045' => '--Sr-Sr-t',
    '7046' => '--Sr-SrwT',
    '7047' => '--Sr-Srwt',
    '7050' => '--Sr-s--T',
    '7051' => '--Sr-s--t',
    '7052' => '--Sr-s-wT',
    '7053' => '--Sr-s-wt',
    '7054' => '--Sr-sr-T',
    '7055' => '--Sr-sr-t',
    '7056' => '--Sr-srwT',
    '7057' => '--Sr-srwt',
    '7060' => '--SrwS--T',
    '7061' => '--SrwS--t',
    '7062' => '--SrwS-wT',
    '7063' => '--SrwS-wt',
    '7064' => '--SrwSr-T',
    '7065' => '--SrwSr-t',
    '7066' => '--SrwSrwT',
    '7067' => '--SrwSrwt',
    '7070' => '--Srws--T',
    '7071' => '--Srws--t',
    '7072' => '--Srws-wT',
    '7073' => '--Srws-wt',
    '7074' => '--Srwsr-T',
    '7075' => '--Srwsr-t',
    '7076' => '--SrwsrwT',
    '7077' => '--Srwsrwt',
    '7100' => '--s--S--T',
    '7101' => '--s--S--t',
    '7102' => '--s--S-wT',
    '7103' => '--s--S-wt',
    '7104' => '--s--Sr-T',
    '7105' => '--s--Sr-t',
    '7106' => '--s--SrwT',
    '7107' => '--s--Srwt',
    '7110' => '--s--s--T',
    '7111' => '--s--s--t',
    '7112' => '--s--s-wT',
    '7113' => '--s--s-wt',
    '7114' => '--s--sr-T',
    '7115' => '--s--sr-t',
    '7116' => '--s--srwT',
    '7117' => '--s--srwt',
    '7120' => '--s-wS--T',
    '7121' => '--s-wS--t',
    '7122' => '--s-wS-wT',
    '7123' => '--s-wS-wt',
    '7124' => '--s-wSr-T',
    '7125' => '--s-wSr-t',
    '7126' => '--s-wSrwT',
    '7127' => '--s-wSrwt',
    '7130' => '--s-ws--T',
    '7131' => '--s-ws--t',
    '7132' => '--s-ws-wT',
    '7133' => '--s-ws-wt',
    '7134' => '--s-wsr-T',
    '7135' => '--s-wsr-t',
    '7136' => '--s-wsrwT',
    '7137' => '--s-wsrwt',
    '7140' => '--sr-S--T',
    '7141' => '--sr-S--t',
    '7142' => '--sr-S-wT',
    '7143' => '--sr-S-wt',
    '7144' => '--sr-Sr-T',
    '7145' => '--sr-Sr-t',
    '7146' => '--sr-SrwT',
    '7147' => '--sr-Srwt',
    '7150' => '--sr-s--T',
    '7151' => '--sr-s--t',
    '7152' => '--sr-s-wT',
    '7153' => '--sr-s-wt',
    '7154' => '--sr-sr-T',
    '7155' => '--sr-sr-t',
    '7156' => '--sr-srwT',
    '7157' => '--sr-srwt',
    '7160' => '--srwS--T',
    '7161' => '--srwS--t',
    '7162' => '--srwS-wT',
    '7163' => '--srwS-wt',
    '7164' => '--srwSr-T',
    '7165' => '--srwSr-t',
    '7166' => '--srwSrwT',
    '7167' => '--srwSrwt',
    '7170' => '--srws--T',
    '7171' => '--srws--t',
    '7172' => '--srws-wT',
    '7173' => '--srws-wt',
    '7174' => '--srwsr-T',
    '7175' => '--srwsr-t',
    '7176' => '--srwsrwT',
    '7177' => '--srwsrwt',
    '7200' => '-wS--S--T',
    '7201' => '-wS--S--t',
    '7202' => '-wS--S-wT',
    '7203' => '-wS--S-wt',
    '7204' => '-wS--Sr-T',
    '7205' => '-wS--Sr-t',
    '7206' => '-wS--SrwT',
    '7207' => '-wS--Srwt',
    '7210' => '-wS--s--T',
    '7211' => '-wS--s--t',
    '7212' => '-wS--s-wT',
    '7213' => '-wS--s-wt',
    '7214' => '-wS--sr-T',
    '7215' => '-wS--sr-t',
    '7216' => '-wS--srwT',
    '7217' => '-wS--srwt',
    '7220' => '-wS-wS--T',
    '7221' => '-wS-wS--t',
    '7222' => '-wS-wS-wT',
    '7223' => '-wS-wS-wt',
    '7224' => '-wS-wSr-T',
    '7225' => '-wS-wSr-t',
    '7226' => '-wS-wSrwT',
    '7227' => '-wS-wSrwt',
    '7230' => '-wS-ws--T',
    '7231' => '-wS-ws--t',
    '7232' => '-wS-ws-wT',
    '7233' => '-wS-ws-wt',
    '7234' => '-wS-wsr-T',
    '7235' => '-wS-wsr-t',
    '7236' => '-wS-wsrwT',
    '7237' => '-wS-wsrwt',
    '7240' => '-wSr-S--T',
    '7241' => '-wSr-S--t',
    '7242' => '-wSr-S-wT',
    '7243' => '-wSr-S-wt',
    '7244' => '-wSr-Sr-T',
    '7245' => '-wSr-Sr-t',
    '7246' => '-wSr-SrwT',
    '7247' => '-wSr-Srwt',
    '7250' => '-wSr-s--T',
    '7251' => '-wSr-s--t',
    '7252' => '-wSr-s-wT',
    '7253' => '-wSr-s-wt',
    '7254' => '-wSr-sr-T',
    '7255' => '-wSr-sr-t',
    '7256' => '-wSr-srwT',
    '7257' => '-wSr-srwt',
    '7260' => '-wSrwS--T',
    '7261' => '-wSrwS--t',
    '7262' => '-wSrwS-wT',
    '7263' => '-wSrwS-wt',
    '7264' => '-wSrwSr-T',
    '7265' => '-wSrwSr-t',
    '7266' => '-wSrwSrwT',
    '7267' => '-wSrwSrwt',
    '7270' => '-wSrws--T',
    '7271' => '-wSrws--t',
    '7272' => '-wSrws-wT',
    '7273' => '-wSrws-wt',
    '7274' => '-wSrwsr-T',
    '7275' => '-wSrwsr-t',
    '7276' => '-wSrwsrwT',
    '7277' => '-wSrwsrwt',
    '7300' => '-ws--S--T',
    '7301' => '-ws--S--t',
    '7302' => '-ws--S-wT',
    '7303' => '-ws--S-wt',
    '7304' => '-ws--Sr-T',
    '7305' => '-ws--Sr-t',
    '7306' => '-ws--SrwT',
    '7307' => '-ws--Srwt',
    '7310' => '-ws--s--T',
    '7311' => '-ws--s--t',
    '7312' => '-ws--s-wT',
    '7313' => '-ws--s-wt',
    '7314' => '-ws--sr-T',
    '7315' => '-ws--sr-t',
    '7316' => '-ws--srwT',
    '7317' => '-ws--srwt',
    '7320' => '-ws-wS--T',
    '7321' => '-ws-wS--t',
    '7322' => '-ws-wS-wT',
    '7323' => '-ws-wS-wt',
    '7324' => '-ws-wSr-T',
    '7325' => '-ws-wSr-t',
    '7326' => '-ws-wSrwT',
    '7327' => '-ws-wSrwt',
    '7330' => '-ws-ws--T',
    '7331' => '-ws-ws--t',
    '7332' => '-ws-ws-wT',
    '7333' => '-ws-ws-wt',
    '7334' => '-ws-wsr-T',
    '7335' => '-ws-wsr-t',
    '7336' => '-ws-wsrwT',
    '7337' => '-ws-wsrwt',
    '7340' => '-wsr-S--T',
    '7341' => '-wsr-S--t',
    '7342' => '-wsr-S-wT',
    '7343' => '-wsr-S-wt',
    '7344' => '-wsr-Sr-T',
    '7345' => '-wsr-Sr-t',
    '7346' => '-wsr-SrwT',
    '7347' => '-wsr-Srwt',
    '7350' => '-wsr-s--T',
    '7351' => '-wsr-s--t',
    '7352' => '-wsr-s-wT',
    '7353' => '-wsr-s-wt',
    '7354' => '-wsr-sr-T',
    '7355' => '-wsr-sr-t',
    '7356' => '-wsr-srwT',
    '7357' => '-wsr-srwt',
    '7360' => '-wsrwS--T',
    '7361' => '-wsrwS--t',
    '7362' => '-wsrwS-wT',
    '7363' => '-wsrwS-wt',
    '7364' => '-wsrwSr-T',
    '7365' => '-wsrwSr-t',
    '7366' => '-wsrwSrwT',
    '7367' => '-wsrwSrwt',
    '7370' => '-wsrws--T',
    '7371' => '-wsrws--t',
    '7372' => '-wsrws-wT',
    '7373' => '-wsrws-wt',
    '7374' => '-wsrwsr-T',
    '7375' => '-wsrwsr-t',
    '7376' => '-wsrwsrwT',
    '7377' => '-wsrwsrwt',
    '7400' => 'r-S--S--T',
    '7401' => 'r-S--S--t',
    '7402' => 'r-S--S-wT',
    '7403' => 'r-S--S-wt',
    '7404' => 'r-S--Sr-T',
    '7405' => 'r-S--Sr-t',
    '7406' => 'r-S--SrwT',
    '7407' => 'r-S--Srwt',
    '7410' => 'r-S--s--T',
    '7411' => 'r-S--s--t',
    '7412' => 'r-S--s-wT',
    '7413' => 'r-S--s-wt',
    '7414' => 'r-S--sr-T',
    '7415' => 'r-S--sr-t',
    '7416' => 'r-S--srwT',
    '7417' => 'r-S--srwt',
    '7420' => 'r-S-wS--T',
    '7421' => 'r-S-wS--t',
    '7422' => 'r-S-wS-wT',
    '7423' => 'r-S-wS-wt',
    '7424' => 'r-S-wSr-T',
    '7425' => 'r-S-wSr-t',
    '7426' => 'r-S-wSrwT',
    '7427' => 'r-S-wSrwt',
    '7430' => 'r-S-ws--T',
    '7431' => 'r-S-ws--t',
    '7432' => 'r-S-ws-wT',
    '7433' => 'r-S-ws-wt',
    '7434' => 'r-S-wsr-T',
    '7435' => 'r-S-wsr-t',
    '7436' => 'r-S-wsrwT',
    '7437' => 'r-S-wsrwt',
    '7440' => 'r-Sr-S--T',
    '7441' => 'r-Sr-S--t',
    '7442' => 'r-Sr-S-wT',
    '7443' => 'r-Sr-S-wt',
    '7444' => 'r-Sr-Sr-T',
    '7445' => 'r-Sr-Sr-t',
    '7446' => 'r-Sr-SrwT',
    '7447' => 'r-Sr-Srwt',
    '7450' => 'r-Sr-s--T',
    '7451' => 'r-Sr-s--t',
    '7452' => 'r-Sr-s-wT',
    '7453' => 'r-Sr-s-wt',
    '7454' => 'r-Sr-sr-T',
    '7455' => 'r-Sr-sr-t',
    '7456' => 'r-Sr-srwT',
    '7457' => 'r-Sr-srwt',
    '7460' => 'r-SrwS--T',
    '7461' => 'r-SrwS--t',
    '7462' => 'r-SrwS-wT',
    '7463' => 'r-SrwS-wt',
    '7464' => 'r-SrwSr-T',
    '7465' => 'r-SrwSr-t',
    '7466' => 'r-SrwSrwT',
    '7467' => 'r-SrwSrwt',
    '7470' => 'r-Srws--T',
    '7471' => 'r-Srws--t',
    '7472' => 'r-Srws-wT',
    '7473' => 'r-Srws-wt',
    '7474' => 'r-Srwsr-T',
    '7475' => 'r-Srwsr-t',
    '7476' => 'r-SrwsrwT',
    '7477' => 'r-Srwsrwt',
    '7500' => 'r-s--S--T',
    '7501' => 'r-s--S--t',
    '7502' => 'r-s--S-wT',
    '7503' => 'r-s--S-wt',
    '7504' => 'r-s--Sr-T',
    '7505' => 'r-s--Sr-t',
    '7506' => 'r-s--SrwT',
    '7507' => 'r-s--Srwt',
    '7510' => 'r-s--s--T',
    '7511' => 'r-s--s--t',
    '7512' => 'r-s--s-wT',
    '7513' => 'r-s--s-wt',
    '7514' => 'r-s--sr-T',
    '7515' => 'r-s--sr-t',
    '7516' => 'r-s--srwT',
    '7517' => 'r-s--srwt',
    '7520' => 'r-s-wS--T',
    '7521' => 'r-s-wS--t',
    '7522' => 'r-s-wS-wT',
    '7523' => 'r-s-wS-wt',
    '7524' => 'r-s-wSr-T',
    '7525' => 'r-s-wSr-t',
    '7526' => 'r-s-wSrwT',
    '7527' => 'r-s-wSrwt',
    '7530' => 'r-s-ws--T',
    '7531' => 'r-s-ws--t',
    '7532' => 'r-s-ws-wT',
    '7533' => 'r-s-ws-wt',
    '7534' => 'r-s-wsr-T',
    '7535' => 'r-s-wsr-t',
    '7536' => 'r-s-wsrwT',
    '7537' => 'r-s-wsrwt',
    '7540' => 'r-sr-S--T',
    '7541' => 'r-sr-S--t',
    '7542' => 'r-sr-S-wT',
    '7543' => 'r-sr-S-wt',
    '7544' => 'r-sr-Sr-T',
    '7545' => 'r-sr-Sr-t',
    '7546' => 'r-sr-SrwT',
    '7547' => 'r-sr-Srwt',
    '7550' => 'r-sr-s--T',
    '7551' => 'r-sr-s--t',
    '7552' => 'r-sr-s-wT',
    '7553' => 'r-sr-s-wt',
    '7554' => 'r-sr-sr-T',
    '7555' => 'r-sr-sr-t',
    '7556' => 'r-sr-srwT',
    '7557' => 'r-sr-srwt',
    '7560' => 'r-srwS--T',
    '7561' => 'r-srwS--t',
    '7562' => 'r-srwS-wT',
    '7563' => 'r-srwS-wt',
    '7564' => 'r-srwSr-T',
    '7565' => 'r-srwSr-t',
    '7566' => 'r-srwSrwT',
    '7567' => 'r-srwSrwt',
    '7570' => 'r-srws--T',
    '7571' => 'r-srws--t',
    '7572' => 'r-srws-wT',
    '7573' => 'r-srws-wt',
    '7574' => 'r-srwsr-T',
    '7575' => 'r-srwsr-t',
    '7576' => 'r-srwsrwT',
    '7577' => 'r-srwsrwt',
    '7600' => 'rwS--S--T',
    '7601' => 'rwS--S--t',
    '7602' => 'rwS--S-wT',
    '7603' => 'rwS--S-wt',
    '7604' => 'rwS--Sr-T',
    '7605' => 'rwS--Sr-t',
    '7606' => 'rwS--SrwT',
    '7607' => 'rwS--Srwt',
    '7610' => 'rwS--s--T',
    '7611' => 'rwS--s--t',
    '7612' => 'rwS--s-wT',
    '7613' => 'rwS--s-wt',
    '7614' => 'rwS--sr-T',
    '7615' => 'rwS--sr-t',
    '7616' => 'rwS--srwT',
    '7617' => 'rwS--srwt',
    '7620' => 'rwS-wS--T',
    '7621' => 'rwS-wS--t',
    '7622' => 'rwS-wS-wT',
    '7623' => 'rwS-wS-wt',
    '7624' => 'rwS-wSr-T',
    '7625' => 'rwS-wSr-t',
    '7626' => 'rwS-wSrwT',
    '7627' => 'rwS-wSrwt',
    '7630' => 'rwS-ws--T',
    '7631' => 'rwS-ws--t',
    '7632' => 'rwS-ws-wT',
    '7633' => 'rwS-ws-wt',
    '7634' => 'rwS-wsr-T',
    '7635' => 'rwS-wsr-t',
    '7636' => 'rwS-wsrwT',
    '7637' => 'rwS-wsrwt',
    '7640' => 'rwSr-S--T',
    '7641' => 'rwSr-S--t',
    '7642' => 'rwSr-S-wT',
    '7643' => 'rwSr-S-wt',
    '7644' => 'rwSr-Sr-T',
    '7645' => 'rwSr-Sr-t',
    '7646' => 'rwSr-SrwT',
    '7647' => 'rwSr-Srwt',
    '7650' => 'rwSr-s--T',
    '7651' => 'rwSr-s--t',
    '7652' => 'rwSr-s-wT',
    '7653' => 'rwSr-s-wt',
    '7654' => 'rwSr-sr-T',
    '7655' => 'rwSr-sr-t',
    '7656' => 'rwSr-srwT',
    '7657' => 'rwSr-srwt',
    '7660' => 'rwSrwS--T',
    '7661' => 'rwSrwS--t',
    '7662' => 'rwSrwS-wT',
    '7663' => 'rwSrwS-wt',
    '7664' => 'rwSrwSr-T',
    '7665' => 'rwSrwSr-t',
    '7666' => 'rwSrwSrwT',
    '7667' => 'rwSrwSrwt',
    '7670' => 'rwSrws--T',
    '7671' => 'rwSrws--t',
    '7672' => 'rwSrws-wT',
    '7673' => 'rwSrws-wt',
    '7674' => 'rwSrwsr-T',
    '7675' => 'rwSrwsr-t',
    '7676' => 'rwSrwsrwT',
    '7677' => 'rwSrwsrwt',
    '7700' => 'rws--S--T',
    '7701' => 'rws--S--t',
    '7702' => 'rws--S-wT',
    '7703' => 'rws--S-wt',
    '7704' => 'rws--Sr-T',
    '7705' => 'rws--Sr-t',
    '7706' => 'rws--SrwT',
    '7707' => 'rws--Srwt',
    '7710' => 'rws--s--T',
    '7711' => 'rws--s--t',
    '7712' => 'rws--s-wT',
    '7713' => 'rws--s-wt',
    '7714' => 'rws--sr-T',
    '7715' => 'rws--sr-t',
    '7716' => 'rws--srwT',
    '7717' => 'rws--srwt',
    '7720' => 'rws-wS--T',
    '7721' => 'rws-wS--t',
    '7722' => 'rws-wS-wT',
    '7723' => 'rws-wS-wt',
    '7724' => 'rws-wSr-T',
    '7725' => 'rws-wSr-t',
    '7726' => 'rws-wSrwT',
    '7727' => 'rws-wSrwt',
    '7730' => 'rws-ws--T',
    '7731' => 'rws-ws--t',
    '7732' => 'rws-ws-wT',
    '7733' => 'rws-ws-wt',
    '7734' => 'rws-wsr-T',
    '7735' => 'rws-wsr-t',
    '7736' => 'rws-wsrwT',
    '7737' => 'rws-wsrwt',
    '7740' => 'rwsr-S--T',
    '7741' => 'rwsr-S--t',
    '7742' => 'rwsr-S-wT',
    '7743' => 'rwsr-S-wt',
    '7744' => 'rwsr-Sr-T',
    '7745' => 'rwsr-Sr-t',
    '7746' => 'rwsr-SrwT',
    '7747' => 'rwsr-Srwt',
    '7750' => 'rwsr-s--T',
    '7751' => 'rwsr-s--t',
    '7752' => 'rwsr-s-wT',
    '7753' => 'rwsr-s-wt',
    '7754' => 'rwsr-sr-T',
    '7755' => 'rwsr-sr-t',
    '7756' => 'rwsr-srwT',
    '7757' => 'rwsr-srwt',
    '7760' => 'rwsrwS--T',
    '7761' => 'rwsrwS--t',
    '7762' => 'rwsrwS-wT',
    '7763' => 'rwsrwS-wt',
    '7764' => 'rwsrwSr-T',
    '7765' => 'rwsrwSr-t',
    '7766' => 'rwsrwSrwT',
    '7767' => 'rwsrwSrwt',
    '7770' => 'rwsrws--T',
    '7771' => 'rwsrws--t',
    '7772' => 'rwsrws-wT',
    '7773' => 'rwsrws-wt',
    '7774' => 'rwsrwsr-T',
    '7775' => 'rwsrwsr-t',
    '7776' => 'rwsrwsrwT',
    '7777' => 'rwsrwsrwt',
  }

  def setup
  end

  def test_permissions_to_octal
    OCTAL_TO_PERMISSIONS.each do |o, p|
      # no header
      assert_equal([o, nil], MiqLinux::Utils.permissions_to_octal(p), "For '#{p}'")

      # directory header
      assert_equal([o, 'dir'], MiqLinux::Utils.permissions_to_octal("d#{p}"), "For 'd#{p}'")

      # link header
      assert_equal([o, 'link'], MiqLinux::Utils.permissions_to_octal("l#{p}"), "For 'l#{p}'")

      # file header
      assert_equal([o, 'file'], MiqLinux::Utils.permissions_to_octal("-#{p}"), "For '-#{p}'")

      # other file header
      assert_equal([o, 'file'], MiqLinux::Utils.permissions_to_octal("c#{p}"), "For 'c#{p}'")
    end
  end

  def test_octal_to_permissions
    OCTAL_TO_PERMISSIONS.each do |o, p|
      # no header
      assert_equal(p, MiqLinux::Utils.octal_to_permissions(o), "For '#{o}'")

      # directory header
      assert_equal("d#{p}", MiqLinux::Utils.octal_to_permissions(o, 'dir'), "For '#{o}', 'dir'")

      # link header
      assert_equal("l#{p}", MiqLinux::Utils.octal_to_permissions(o, 'link'), "For '#{o}', 'link'")

      # file header
      assert_equal("-#{p}", MiqLinux::Utils.octal_to_permissions(o, 'file'), "For '#{o}', 'file'")

      # character header
      assert_equal("d#{p}", MiqLinux::Utils.octal_to_permissions(o, 'd'), "For '#{o}', 'd'")

      # numeric octal no header
      assert_equal(p, MiqLinux::Utils.octal_to_permissions(o.to_i(8)), "For '#{o}' (numeric)")

      # numeric octal directory header
      assert_equal("d#{p}", MiqLinux::Utils.octal_to_permissions(o.to_i(8), 'dir'), "For '#{o}' (numeric), 'dir'")
    end
  end
end
